/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;

import java.awt.event.ContainerListener;
import java.awt.event.ContainerEvent;
import java.util.Vector;
import java.lang.IllegalArgumentException;

import common.*;

/**
 * DGS Bordered Panel
 * DBPanel is a panel component that has an optional border
 * and optional text title.
 */

public class DBPanel extends Panel implements ContainerListener, Bevel
{
    public static final int BEVEL_NONE = 0;
    public static final int BEVEL_LINE = 1;
    public static final int BEVEL_LOWERED = 2;
    public static final int BEVEL_RAISED = 3;


    public static final int ALIGN_LEFT = 0;
    public static final int ALIGN_CENTERED = 1;
    public static final int ALIGN_RIGHT = 2;


    /**
     * Constructs a new default DBPanel.
     * Its label will be ALIGN_CENTERED and it will have a BEVEL_LINE style border.
     */
	public DBPanel() { this(null, ALIGN_CENTERED, BEVEL_LINE); }

    /**
     * Constructs a new DBPanel with the specified beveled border style.
     * It will have the ALIGN_CENTERED label alignment style.
     */
	public DBPanel(int style) { this(null, ALIGN_CENTERED, style); }

    /**
     * Constructs a new DBPanel with the specified border label.
     * Its label will be ALIGN_LEFT and it will have a BEVEL_LINE style border.
     */
	public DBPanel(String s) { this(s, ALIGN_LEFT, BEVEL_LINE); }

    /**
     * Constructs a new DBPanel with the specified border label
     * and given alignment.
     */
	public DBPanel(String s, int alignment)	{ this(s, alignment, BEVEL_LINE);}

    /**
     * Constructs a new DBPanel with the specified border label
     * and given alignment and border styles.
     */

	public DBPanel(String s, int alignment, int style)
	{
	    borderColor	= Color.black;
	    labelColor  = Color.black;
	    padleft     = 2;
	    padright    = 2;
	    padtop      = 2;
	    padbottom   = 2;
	    ixPadLeft   = 2;
	    ixPadRight  = 2;	    
	    iyPadTop    = 2;
	    iyPadBottom = 2;

	    label = (s != null && s.length() == 0) ? null : s;
	    labelAlignment = alignment;
	    internalInsets = new Insets(2, 2, 2, 2);
	    this.style     = style;
	    super.setLayout(null);
	    super.add(panel = new Panel());
	    sizepanel(true);

        cListeners = new Vector();

        setLayout(null);  // Layout de panel
        
	}
	

    /**
     * Sets the borders outside padding amount, in pixels.
     * This is the distance between the drawn border and the actual
     * bounds of the component.
     */

	public void setPadding(int top, int bottom, int left, int right)
	{
	    padtop = top;
	    padbottom = bottom;
		padleft = left;
	    padright = right;

	    sizepanel(true);
	    invalidate();
	}


    /**
     * Sets the borders inset padding amount, in pixels.
     * This is the distance between the drawn border and the usable
     * area within the border. It is used to determine the size of the
     * panel contained within this component's borders.
     */


    public void setIPadding(int l, int t, int r, int b)
    {
       ixPadLeft = l;
       ixPadRight = r;       
       iyPadTop = t;
       iyPadBottom = b;
       
       sizepanel(true);
	   invalidate();
           
    }    


    /**
     * Sets the internal border insets.
     * The left and right fields of the internal insets are used while
     * determining the preferred width of this component.
     */
	public void setInternalInsets(Insets newInsets)
	{
	    internalInsets = newInsets;
	    sizepanel(true);
	    invalidate();
	}

	public Insets getInternalInsets()
	{
	    return internalInsets;
	}


    /**
     * Sets the text label to display in the border.
     */
	public void setLabel(String newLabel)
	{

		if (newLabel != null && newLabel.length() == 0)	newLabel = null;

	    label = newLabel;
	    sizepanel(true);
	    //invalidate();
	    repaint();
	}


	/**
	 * Sets the current border color.
	 */
	public void setBorderColor(Color newBorderColor)
	{
		borderColor	= newBorderColor;
		repaint();
	}


	/**
	 * Sets the border label color.
	 */
	public void setLabelColor(Color newLabelBorderColor)
	{
	    labelColor = newLabelBorderColor;
		repaint();
	}

    /**
     * Sets the border label alignment.
     * One of ALIGN_LEFT, ALIGN_CENTERED, or ALIGN_RIGHT
     */
	public void setAlignStyle(int newLabelAlignment)
	{
	    labelAlignment = newLabelAlignment;
	    sizepanel(true);
	    repaint();
	}

    /**
     * Sets the border style.
     * One of BEVEL_RAISED, BEVEL_LOWERED, BEVEL_LINE, or BEVEL_NONE
     */
	public void setBevelStyle(int newBevelStyle)
	{
	    style = newBevelStyle;
	    repaint();
	}



	/**
	 * Returns the recommended dimensions to properly display this component.
	 */
	public Dimension getPreferredSize()
	{
    	Dimension p = getSize();
    	Dimension m = getMinimumSize();
    	return new Dimension(Math.max(p.width, m.width), Math.max(p.height, m.height));
	}


	/**
	 * Returns the minimum dimensions to properly display this component.
	 */
	public Dimension getMinimumSize()
	{
	    return new Dimension(20, 40);
	}


	/**
	 * Sets the layout manager to be used to layout this container.
	 **/

	public void setLayout(LayoutManager l)
	{
     if (panel != null)	panel.setLayout(l);
	}


    /**
     * Gets the component at the specified zero-relative component index.
     */
	public Component getComponent(int i)
	{
	    return panel.getComponent(i);
	}

	/**
	 * Returns all of the components in this container.
     */
	public Component[] getComponents()
	{
	    return panel.getComponents();
	}

	/**
	 * Returns the number of components in this container.
     */
	public int getComponentCount()
	{
	    return panel.getComponentCount();
	}


	/**
	 * Tells this component that it has been added to a container.
	 * It has been overridden here to hook-up event listeners, and
	 * to resize self after peer created.
	 */
	public synchronized void addNotify()
	{
		super.addNotify();
		sizepanel(true);

		//Hook up listeners
		if (panel != null)
	     panel.addContainerListener(this);
	}

	/**
	 * Tells this component that it is being removed from a container.
	 * This is a standard Java AWT method which gets called by the AWT when
	 * this component is removed from a container. Typically, it is used to
	 * destroy the peers of this component and all its subcomponents.
	 *
	 * It has been overridden here to unhook event listeners.
	 *
	 * @see #addNotify
	 */
	public synchronized void removeNotify()
	{
		//Unhook listeners

		if (panel != null)
		    panel.removeContainerListener(this);

		super.removeNotify();
	}

	/**
     * Handles the laying out of components within this component.
     * This is a standard Java AWT method which gets called by the AWT
     * when this component is validated with the validate() method.
     *
     * @see java.awt.Container#validate
	 */
	public void doLayout()
	{
	    sizepanel(false);
	    panel.doLayout();
	}

    /**
     * Adds the specified component to this container at the specified
     * index.  Also notifies the layout manager to add the component to
     * the this container's layout using the specified constraints object.
     * <p>
     * This is the method to override if you want to track every add
     * request to a container.  An overriding method should usually
     * include a call to super.addImpl(comp, constraints, index).
     *
     * DBPanel overrides this method to send all additions to it to\
     * its internal panel
     */
	protected void addImpl(Component comp, Object constraints, int index)
	{
	    if (comp == panel)
			super.addImpl(comp,constraints,index);
		else
			panel.add(comp,constraints,index);
	}

	/**
	 * Removes the component at the specified index from this container.
	 */

	public void remove(int index) { panel.remove(index); }
	public void remove(Component comp) { panel.remove(comp); }

	/**
	 * Removes all the components from this container.
	 * This is a standard Java AWT method which gets called to remove all
	 * the components from a container. When this happens each component's
	 * removeNotify() will also get called to indicate component removal.
	 */
	public void removeAll()
	{
	    panel.removeAll();
	}

	/**
	 * Moves and/or resizes this component.
	 * This is a standard Java AWT method which gets called to move and/or
	 * resize this component. Components that are in containers with layout
	 * managers should not call this method, but rely on the layout manager
	 * instead.
	 */
	public void setBounds(int x, int y, int width, int height)
	{
	    super.setBounds(x, y, width, height);
	    sizepanel(false);
	}


    /**
     * Handles redrawing of this component on the screen.
     * This is a standard Java AWT method which gets called by the Java
     * AWT (repaint()) to handle repainting this component on the screen.
     * The graphics context clipping region is set to the bounding rectangle
     * of this component and its [0,0] coordinate is this component's
     * top-left corner.
     * Typically this method paints the background color to clear the
     * component's drawing space, sets graphics context to be the foreground
     * color, and then calls paint() to draw the component.
     */

	public void update(Graphics g)
	{
	    Dimension s;
	    Insets insets;

	    s      = getSize();
	    insets = getInsets();

	    g.setColor(getBackground());

	    if (insets.left > 0)
	        g.fillRect(0, 0, insets.left, s.height);

	    if (insets.top > 0)
	        g.fillRect(0, 0, s.width, insets.top);

	    if (insets.bottom > 0)
	        g.fillRect(0, s.height-insets.bottom, s.width, insets.bottom);

	    if (insets.right > 0)
	        g.fillRect(s.width-insets.right, 0, insets.right, s.height);

	    paint(g);
	    panel.repaint();
	}

	/**
	 * Returns the number of components in this container.
	 */

	public int getComponentsCount()
	{
	    return panel.getComponentCount();
	}

    /**
     * Returns the amount of space used by the current border.
     */
     
	public Insets getInsets()
	{
	    int h = getLabelTopMargin();
	    Insets insets = getInternalInsets();

	    return new Insets(h + insets.top, insets.left, insets.bottom, insets.right);
	}

	/**
	 * Sets this component's background color.
	 * This is a standard Java AWT method which gets called to change
	 * the background color of this component.
	 */
	public void setBackground(Color c)
	{
	    super.setBackground(c);
	}

	public void setInnerBackground(Color c)
	{
	    panel.setBackground(c);
	}

	public void componentAdded(ContainerEvent e)
	{
	    if (e.getSource() == panel)
	    {
	        for (int i = 0; i < cListeners.size(); ++i)
	        {
	            ((ContainerListener) cListeners.elementAt(i)).componentAdded(
	                new ContainerEvent(this, ContainerEvent.COMPONENT_ADDED, e.getChild()));
	        }
	    }
	}


	public void componentRemoved(ContainerEvent e)
	{
	    if (e.getSource() == panel)
	    {
	        for (int i = 0; i < cListeners.size(); ++i)
	        {
	            ((ContainerListener) cListeners.elementAt(i)).componentRemoved(
	                new ContainerEvent(this, ContainerEvent.COMPONENT_REMOVED, e.getChild()));
	        }
	    }
	}

	public void addContainerListener(ContainerListener l)
	{
	    if (panel != null)
	    {
	        panel.addContainerListener(l);
	        cListeners.addElement(l);
	    }
	}

	public void removeContainerListener(ContainerListener l)
	{
	    if (panel != null)
	    {
	        panel.removeContainerListener(l);
	        cListeners.removeElement(l);
	    }
	}

  
    /**
     * Sizes the border panel from the padding, inset, and label margin information.
     * @param force force setBounds even if the overall size hasn't changed
     */
	protected void sizepanel(boolean force)
	{
	    Dimension s;

	    s = getSize();

	    if (force || oldSize.width != s.width || oldSize.height != s.height)
	    {
	        oldSize = s;
	        panel.setBounds(padleft + ixPadLeft,
	              getLabelTopMargin() + iyPadTop,
	              s.width - padright - padleft - ixPadLeft - ixPadRight - 1,
	              s.height - padbottom - getLabelTopMargin() - iyPadBottom - iyPadTop - 1);
	    }
	}

	/**
	 * Draws the border panel.
	 */
	protected void draw(Graphics g)
	{
		Dimension s;
		int delta;
		FontMetrics fm;
		int x;
		int y;
		int w;
		int h;

		s = getSize();
		delta = padtop;
		fm = getFontMetrics(getFont());

		//g.clipRect(0, 0, s.width, s.height);

		if (label != null && fm != null)
		{
			delta = (fm.getAscent()  + fm.getDescent() + padtop) / 2;
		}

		x = padleft;
		y = delta;
		w = s.width - padleft - padright - 1;
		h = s.height - 1 - delta - padbottom;

        
		drawBorder(g, x, y, w, h);
		drawLabel(g, fm);
	}

    /**
     * Draws the border panel border.
     * @param g current graphics object
     * @param x y w h coordinate of panel object
     */
	protected void drawBorder(Graphics g, int x, int y, int w, int h)
	{
	    switch(style)
	    {
	        case BEVEL_RAISED :
	        {
	            g.setColor(bevelLighterColor);
	            g.drawLine(x, y, x + w, y);
	            g.drawLine(x, y, x, y + h);
	            g.setColor(bevelDarkerColor);
	            g.drawLine(x, y + h, x + w, y + h);
	            g.drawLine(x + w, y, x + w, y + h);
	            break;
	        }
	        case BEVEL_LOWERED :
	        {
	            g.setColor(bevelDarkerColor);
	            g.drawLine(x, y, x + w, y);
	            g.drawLine(x, y, x, y + h);
	            g.setColor(bevelLighterColor);
	            g.drawLine(x, y + h, x + w, y + h);
	            g.drawLine(x + w, y, x + w, y + h);
	            break;
	        }
	        case BEVEL_LINE :
	        {
	            g.setColor(borderColor);
	            g.drawRect(x, y, w, h);
	            break;
	        }
	        case BEVEL_NONE :
	        default: break;
	    }
	}

	/**
	 * Draws the border panel label.
	 * @param g current graphics object
	 * @param fm FontMetrics of border panel label
	 */
	protected void drawLabel(Graphics g, FontMetrics fm)
	{
	    if (label != null && (fm != null))
	    {
	        int fWidth;
	        Dimension s;
	        int stringWidth;
	        int ascent;
	        int descent;
	        int x;
	        int y;
	        int h;

	        fWidth = 10;
	        s = getSize();

	        if (getFont().getSize() > fWidth)
	        {
	            fWidth = fWidth + getFont().getSize() / 2;
	        }

	        stringWidth = fm.stringWidth(label);
	        ascent      = fm.getAscent();
	        descent     = fm.getDescent();

	        switch(labelAlignment)
	        {
	         case Label.CENTER:
	           x = (s.width - stringWidth) / 2;
	           break;
             case Label.RIGHT:
               x = s.width - fWidth - (stringWidth + (labelpadx + labelipadx) / 2);
	           break;
             case Label.LEFT:   {  }
             default:
               x = fWidth + (labelpadx + labelipadx) / 2;
	        }

	        h = ascent + descent+padtop;
	        y = (fWidth - h) / 2 + (padtop + ascent);

	        g.setColor(getBackground());
	        g.fillRect(x - labelipadx / 2, y - 1 - ascent - padtop/2, stringWidth + labelipadx, h);
	        g.setColor(labelColor);
	        g.drawString(label, x, y - 1);
	    }
	}


	/**
	 * Paints this component using the given graphics context.
	 */

	public void paint(Graphics g)
	{
	    sizepanel(false);
		Color curBackground = getBackground();
		//cachedBackground = curBackground;
		calculateHilightColors(curBackground);
	    g.setColor(curBackground);
	    draw(g);
	}


    /**
     * Returns the current margin above the label.
     */
	protected int getLabelTopMargin()
	{
	    int  top;
	    Font font;

	    if (label == null) return padtop;

	    top  = padtop;
	    font = getFont();

	    if (font != null)
	    {
	      FontMetrics fm;
	      fm  = getFontMetrics(font);
	      top = fm.getAscent() + fm.getDescent() + padtop + 0;
	    }
      return top;
	}

	/**
	 * Returns the current label width margin.
	 */
	protected int getLabelWidthMargin()
	{
	    if (label == null)  return 0;

	    int w;
	    Font font;

	    w = 2 + internalInsets.left + internalInsets.right;
	    font = getFont();

	    if (font != null)
	    {
	        FontMetrics fm;

	        fm = getFontMetrics(font);
	        w  = Math.max(w, 2 + fm.stringWidth(label) + labelpadx + labelipadx);
	    }

	    return w;
	}

	/**
	 * Used to calculate the hilight colors from the background color.
	 */
	protected void calculateHilightColors(Color c)
	{
		bevelLighterColor	= ColorUtils.calculateHilightColor(c);
		bevelDarkerColor	= ColorUtils.calculateShadowColor(c);
	}

    /**
     * The overall size before sizepanel().
     */
    Dimension oldSize = new Dimension();

    /**
     * Label horizontal coordinate padding constant, in pixels.
     */
    protected static final int labelpadx = 10;
    /**
     * Label horizontal coordinate inset constant, in pixels.
     */
    protected static final int labelipadx = 4;

    /**
     * Cached value of the background color.  Used to determine if calculated colors need to be updated.
     */
	//protected Color cachedBackground	= null;
    /**
     * The border color.
     */
    protected Color borderColor;
    /**
     * The color to use as a hilight when BevelStyle is BEVEL_RAISED or BEVEL_LOWERED.
     */
    protected Color bevelLighterColor;
    /**
     * The color to use as a hilight when BevelStyle is BEVEL_RAISED or BEVEL_LOWERED.
     */
    protected Color bevelDarkerColor;
    /**
     * The border label color.
     */
    protected Color labelColor;
    /**
     * The top border outside padding amount, in pixels.
     * This is the distance between the drawn border and the actual
     * bounds of the component.
     */
    protected int padtop;
    /**
     * The bottom border outside padding amount, in pixels.
     * This is the distance between the drawn border and the actual
     * bounds of the component.
     */
    protected int padbottom;
    /**
     * The left border outside padding amount, in pixels.
     * This is the distance between the drawn border and the actual
     * bounds of the component.
     */
    protected int padleft;
    /**
     * The right border outside padding amount, in pixels.
     * This is the distance between the drawn border and the actual
     * bounds of the component.
     */
    protected int padright;
	/**
     * The side border inset padding amount, in pixels.
     * This is the distance between the drawn border and the usable
     * area within the border. It is used to determine the size of the
     * panel contained within this component's borders.
     * @see #panel
     */
	protected int ixPadLeft;
	protected int ixPadRight;	
    /**
     * The top border inset padding amount, in pixels.
     * This is the distance between the drawn border and the usable
     * area within the border. It is used to determine the size of the
     * panel contained within this component's borders.
     * @see #panel
     */
    protected int iyPadTop;
    /**
     * The bottom border inset padding amount, in pixels.
     * This is the distance between the drawn border and the usable
     * area within the border. It is used to determine the size of the
     * panel contained within this component's borders.
     * @see #panel
     */
    protected int iyPadBottom;
    /**
     * The border style: BEVEL_RAISED, BEVEL_LOWERED, BEVEL_LINE, or BEVEL_NONE.
     */
    protected int style;
    /**
     * The text label to display in the border.
     */
    protected String label;
    /**
     * The border label alignment: ALIGN_LEFT, ALIGN_CENTERED, or ALIGN_RIGHT.
     */
    protected int labelAlignment;
    /**
     * The internal border insets.
     * The left and right fields are used while determining the preferred
     * width of this component.
     */
    protected Insets internalInsets;
    /**
     * The panel within the border. This is the panel that contains the
     * components added to the DBPanel.
     */
    protected Panel panel;

	private Vector cListeners = null;

}
