/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;

/**
   Displays a string in a rectangle with a border
 */
public class DLabel extends Component
{
    public static final int BEVEL_LOWERED = 3;
    public static final int BEVEL_RAISED = 2;
    public static final int BEVEL_LINE = 1;
    public static final int BEVEL_NONE = 0;

    public static final int ALIGN_LEFT = 0;
    public static final int ALIGN_CENTER = 1;
    public static final int ALIGN_RIGHT = 2;

    /**
     * Constructs an empty Label with default setting
     */
    public DLabel()
    {
        this("", BEVEL_NONE, ALIGN_LEFT, Color.black);
    }

    public DLabel(int bevel)
    {
        this("", bevel, ALIGN_LEFT, Color.black);
    }

    /**
     * Constructs a DLabel
     */
    public DLabel(String sText)
    {
       this(sText, BEVEL_NONE, ALIGN_LEFT, Color.black);
    }

    /**
     * Constructs a DLabel with bevel
     */
    public DLabel(String sText, int bevelStyle)
    {
        this(sText, bevelStyle, ALIGN_LEFT, Color.black);
    }

    /**
     * Constructs a DLabel with bevel and alignment
     */
    public DLabel(String sText, int bevelStyle, int alignStyle)
    {
        this(sText, bevelStyle, alignStyle, Color.black);
    }


    /**
     * Constructs a DLabel with all attributes specified.
     */
    public DLabel(String sText, int bevelStyle, int alignStyle, Color color)
    {
		sLabel = sText;
		textColor = color;
		borderColor = Color.black;
		cachedBackground = getBackground();

		this.alignStyle = alignStyle;
		this.bevelStyle = bevelStyle;
    }


    /**
     * Sets the text alignment
     */
    public void setAlignment(int a)
    {
        alignStyle = a;
        repaint();
    }

    /**
     * Sets the border style.
     */
    public void setBevel(int b)
    {
	   bevelStyle = b;
	   repaint();
    }

    /**
     * Sets the border color.
     */
    public void setBorderColor(Color bc)
    {
       borderColor = bc;
    }

    /**
     * Sets the label text.
     */
    public void setText(String newText)
    {
      sLabel = newText;
      repaint();
    }

    /**
     * Gets the current label text.
     */
    public String getText() { return sLabel;}

    /**
     * Sets the label text color.
     */
    public void setTextColor(Color tc)
    {
		textColor = tc;
    }

    public void setStyle(Style s)
    {
     setFont(s.font);
     setBackground(s.background);
     textColor = s.foreground;
    }


    public void update(Graphics g) { paint(g); }

    public void paint(Graphics g)
    {
        Color color1, color2;

	Color curBackground = getBackground();
	if (!Tool.objectsEqual(curBackground, cachedBackground))
	{
	 cachedBackground = curBackground;
	 hilightColor	= Effect.hilight(curBackground);
	 shadowColor	= Effect.shadow(curBackground);
	}

        Rectangle r = getBounds();

	g.setFont(getFont());

       switch (bevelStyle)
       {
        case BEVEL_LOWERED :
            color1 = shadowColor;
            color2 = hilightColor;
            break;
        case BEVEL_RAISED :
            color1 = hilightColor;
            color2 = shadowColor;
            break;
        case BEVEL_LINE :
            color1 = borderColor;
            color2 = borderColor;
            break;
        default:
            color1 = color2 = null;
            break;
       }

        g.setColor(getBackground());
        g.fillRect(0, 0, r.width - 1, r.height - 1);

        if (color1 != null)
        {
          // top
          g.setColor(color1);
          g.drawLine(0, 0, r.width-1, 0);

          // bottom
          g.setColor(color2);
          g.drawLine(0, r.height-1, r.width-1, r.height-1);

          // left
          g.setColor(color1);
          g.drawLine(0, 0, 0, r.height-1);

          // right
          g.setColor(color2);
          g.drawLine(r.width-1, 0, r.width-1, r.height-1);
        }

        // text
        g.setColor(textColor);

        fm = getFontMetrics(g.getFont());
        yTemp = (r.height - 1 + fm.getAscent()) / 2 - 2;

        switch (alignStyle)
        {
            default:
            case ALIGN_LEFT:
                if (bevelStyle == BEVEL_LINE)
                    g.drawString(sLabel, 4, yTemp);
                else
                    g.drawString(sLabel, 8, yTemp);

                break;
            case ALIGN_RIGHT:
                xTemp = r.width - fm.stringWidth(sLabel);
                if (bevelStyle == BEVEL_LINE)
                    g.drawString(sLabel, xTemp - 6, yTemp);
                else
                    g.drawString(sLabel, xTemp - 10, yTemp);
                break;
            case ALIGN_CENTER:
                xTemp = (r.width - fm.stringWidth(sLabel)) / 2;
                g.drawString(sLabel, xTemp, yTemp);
                break;
        }

    }

    /**
	 * Returns the recommended dimensions to properly display this component.
     *
     * For each axis, the preferred size is the maximum of the current size
     * and the minimum size needed to display the entire label text.
	 */
    public Dimension getPreferredSize()
    {
        Dimension s = getSize();
        Dimension m = getMinimumSize();

        return new Dimension(Math.max(s.width, m.width), Math.max(s.height, m.height));
    }


    /**
	 * Returns the minimum dimensions to properly display this component.
     * The minimum size is that size needed to display the entire label text.
	 */
    public Dimension getMinimumSize()
    {
        Dimension min;
        Font f;

        min = new Dimension(16, 8);
        f   = getFont();

        if (f != null)
        {
            fm = getFontMetrics(f);
            min.width = fm.stringWidth(sLabel) + 16;
            min.height = fm.getHeight() + 8;
        }

        return min;
    }



    /**
     * The label text.
     */
    protected String sLabel;

	/**
     * The current text alignment style.
     */
    protected int alignStyle;
    /**
     * The current border bevel style.
     */
    protected int bevelStyle;
    /**
     * The label text color.
     */
    protected Color textColor;
    /**
     * The border color.
     */
    protected Color borderColor;
    /**
     * The color of border bevel hilights.
     * This color is automatically determined.
     */
    protected Color hilightColor;
    /**
     * The color of border bevel shawdows.
     * This color is automatically determined.
     */
    protected Color shadowColor;
    /**
     * Cached value of the background color.  Used to determine
     * if calculated colors need to be updated.
     */
	protected Color cachedBackground	= null;

    transient private FontMetrics fm;
    transient private int xTemp;
    transient private int yTemp;

}

