/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;
/**
     DListBox
     Added to classic listbox:
     - requestFocus at mouse management to let KeyAdapter working.
     - tagged items
     - adding an unique item
     - various functions.
*/

import java.awt.*;
import java.util.Vector;
import java.awt.image.MemoryImageSource;
import java.awt.event.*;
import java.net.URL;
import common.*;

/**
 * A box containing a list of images and/or text.
 * Use an ListBox to display a set of images and/or text that the user
 * can select.
 * The user cannot enter text in an ListBox.
*/
public class DListBox extends Panel implements java.awt.ItemSelectable, KeyListener, MouseListener, java.awt.event.AdjustmentListener
{
    /** The ID of the event posted when the image portion of a list item is selected  */
    public static final int EVT_IMAGE_SELECT = 0x4004;

    /** Defines border styles */
    public static final int BORDER_REGULAR = 0;
    public static final int BORDER_NONE = 1;

    /** Defines the width images displayed in the list, in pixels */
    public static final int IMAGE_WIDTH = 20;

    private static final boolean ABSOLUTE = true;

    // variables sources des lments
    public Image image = null;    // Pointeur sur l'image
    public String[] arrayModel;   // Pointeur sur le tableau

    /** DGS */
    boolean listChanged;   // quand on modifie le contenu


    /**
     * Constructs a default DListBox.
     * - will size to show all of the existing list items when automatically laid out,
     * - and allows single list item selection.
     */
    public DListBox() { this(-1, false); }

    /**
     * Constructs an DListBox with the specified label and
     * conditionally allows multiple selections.
     * It will size to show all of the existing list items when
     * automatically laid out.
     * Note: the label of an DListBox does not get displayed.
     * @param bMultipleSelections if true then multiple selections are allowed;
     * if false only single item selections are allowed
     */
    public DListBox(boolean bMultipleSelections)
    {
        this(-1, bMultipleSelections);
    }


    /**
     * Constructs an DListBox with the specified number of visible rows,
     * and conditionally allows multiple list item selection.
     * @param rows the number of row items to show.
     * If rows is less than or equal to zero,
     * it will size to show all of the existing list items when
     * automatically laid out.
     * @param bMultipleSelections if true then multiple selections are allowed;
     * if false only single item selections are allowed
     */
    public DListBox(int rows, boolean bMultipleSelections)
    {
        items = new Vector();

        rowsToShow = rows;
        this.bMultipleSelections = bMultipleSelections;

        setLayout(null);

        VBar = new Scrollbar();
        VBar.setBackground(SystemColor.control);
        VBar.setVisible(false);
        add(VBar);

        HBar = new Scrollbar(Scrollbar.HORIZONTAL);
        HBar.setBackground(SystemColor.control);
        HBar.setVisible(false);
        add(HBar);

	cachedBackground = getBackground();	// Initialize the cached color.

        //if(OS.isWindows())
        //  barSize  = 17;
        listChanged = false;

        //parent = this;
        //add(pop); // add popup menu
        addKeyListener(this);
        addMouseListener(this);
     }


     public boolean isFocusable() { return true; }

     /** DGS
        Affecte l'image par dfaut
       */
     public void setImage(Image img)  {  this.image = img; }

     /** DGS
        Set the data array from a string array
       */
     public void setModel(String[] model) { this.arrayModel = model; }


    /**
     * Sets the vertical scrollbar visibility flag.
     * @param cond if true, the vertical scrollbar will be made
     * visible when necessary; if false, the vertical scrollbar
     * will never be made visible
     */
    public void setShowVerticalScroll(boolean cond)
    {
    	if(bAllowShowVBar != cond)
    	{
            bAllowShowVBar = cond;
            invalidate();
	}
    }

    /**
     * Gets the current vertical scrollbar visibility flag.
     * @returns true if the vertical scrollbar will be made
     * visible when necessary; false if the vertical scrollbar
     * will never be made visible
     */
    public boolean isShowVerticalScroll()
    {
      return bAllowShowVBar;
    }

    /**
     * Sets the horizontal scrollbar visibility flag.
     * @param cond if true, the horizontal scrollbar will be made
     * visible when necessary; if false, the horizontal scrollbar
     * will never be made visible
     */
    public void setShowHorizontalScroll(boolean cond)
    {
    	if(bAllowShowHBar != cond)
    	{
            bAllowShowHBar = cond;
            invalidate();
	}
    }

    /**
     * Gets the current horizontal scrollbar visibility flag.
     * @returns true if the horizontal scrollbar will be made
     * visible when necessary; false if the horizontal scrollbar
     * will never be made visible
     */
    public boolean isShowHorizontalScroll()
    {
      return bAllowShowHBar;
    }


    /**
     * Sets DListBox "ComboBox mode".
     * "ComboBox mode" allows the selection point to follow
     * the mouse even when the mouse button is not down.
     * This only applies when the DListBox is not allowing multiple
     * selections.
     */
    public void setComboMode(boolean cond)
    {
    	if(bComboMode != cond)
    	{
          bComboMode = cond;
          yAdj = 0;
          if(OS.isWindows()) yAdj = 6;
          invalidate();
	}
    }

    /**
     * Gets the current "ComboBox mode" setting.
     * "ComboBox mode" allows the selection point to follow
     * the mouse even when the mouse button is not down.
     * This only applies when the DListBox is not allowing multiple
     * selections.
     * @return true if in "ComboBox mode", otherwise false
     * @see #setComboMode
     */
    public boolean isComboMode() { return bComboMode;}


    /**
     * Sets the number of rows to display in the DListBox.
     * This affects the dimensions returned by getPreferredSize and determines
     * the number of rows displayed when this component is automatically laid out.
     */
    public void setRowsToShow(int rows)
    {
    	if(rowsToShow != rows)
    	{
	  rowsToShow = rows;
	  invalidate();
    	}
    }

    /**
     * Gets the current number of rows to display.
     * This affects the dimensions returned by getPreferredSize and determines
     * the number of rows displayed when this component is automatically laid out.
     * @return the current number of rows to automatically display
     */
    public int getRowsToShow()  { return rowsToShow; }


    /**
     * Sets the border type of the DListBox.
     * A regular border makes the list box appear lower than the surrounding area.
     * @param type new border type, BORDER_REGULAR or BORDER_NONE
     */
    public void setBorderType(int type)
    {
      if(borderType != type)
      {
        borderType = type;
        if (type == BORDER_REGULAR)
        {
            borderWidth = 4;
            halfBorderWidth = 2;
        }
        else
        {
            borderWidth = 0;
            halfBorderWidth = 0;
        }
        invalidate();
      }
    }

    /**
     * Gets the current border type of the DListBox.
     * A regular border makes the list box appear lower than the surrounding area.
     * @return the current border type, BORDER_REGULAR or BORDER_NONE
     * @see #setBorderType
     */
    public int getBorderType() { return borderType; }

    /**
     * Conditionally show a border around the cell at the specified index.
     * Cell borders are always shown as a solid single-pixel wide line around
     * the cell.
     * @param index the zero-relative index of the cell
     * @param bOn true to show the cell borders; false to not show them
     * @see #setCellBorders
     */
    public void setCellBorder(int index, boolean bOn)
    {
        if (validIndex(index))
        {
            ListItem li = (ListItem) items.elementAt(index);
            if (li.bCellBorder != bOn)
            {
                li.bCellBorder = bOn;
                li.bDirty = true;
                invalidate();
            }
        }
    }

    /**
     * Sets the DListBox cell border display default, and
     * resets all items to have the border conditionally.
     * Cell borders are always shown as a solid single-pixel wide line around
     * the cell.
     * @param bOn true for "grid lines on by default" mode,
     * false for "grid lines off by default" mode.
     * @see #setCellBorder
     * @see #isCellBorders
     */
    public void setCellBorders(boolean bOn)
    {
    	if(bCellBorders != bOn)
    	{
	        bCellBorders = bOn;

	        int s = items.size();
	        ListItem li = null;

	        for (int x = 0; x < s; x++)
	        {
	            li = (ListItem) items.elementAt(x);
	            li.bCellBorder = bOn;
	        }
	        bAllDirty = true;
	        invalidate();
		}
    }

    /**
     * Gets the current default cell border display mode.
     * Cell borders are always shown as a solid single-pixel wide line around
     * the cell.
     * @return true if the "grid lines on by default" mode is on, otherwise false
     * @see #setCellBorders
     */
    public boolean isCellBorders()  {  return bCellBorders; }

    /**
     * Sets the text color for an item at the given index.
     * @param index the zero-relative index of the item
     * @param color the color, null to use the default text color
     */
    public void setEnabledTextColor(int index, Color color)
    {
      if (validIndex(index))
       {
         ListItem tempItem = ((ListItem)items.elementAt(index));
	     tempItem.color = color;
       }
    }

    /** DGS
       Barrer l'item
      */

    public void crossOut(int index)
    {
      if (validIndex(index))
       {
         ListItem tempItem = ((ListItem)items.elementAt(index));
         tempItem.deleted = true;
         tempItem.bDirty = true;
       }
    }

    public void crossOut(int index, boolean val)
    {
      if (validIndex(index))
       {
         ListItem tempItem = ((ListItem)items.elementAt(index));
         tempItem.deleted = val;
         tempItem.bDirty = true;
      }
    }


    public boolean isCrossedOff(int index)
    {
      if (validIndex(index))
       {
        ListItem tempItem = ((ListItem)items.elementAt(index));
	    return(tempItem.deleted);
       }
      return false;
    }


    /** DGS
        Retourne le numro du fichier barr
     */

    public int getCrossedOffIndex()
    {
      int l = items.size();

      for(int index = 0; index < l; index++)
       {
         ListItem tempItem = ((ListItem)items.elementAt(index));
	     if(tempItem.deleted) return index;
       }
      return -1;
    }



    /** DGS
      S'il existe au moins un fichier barr
     */

    public boolean isCrossedOff()
    {
      return (getCrossedOffIndex() != -1);
    }



    /**
     * Gets the text color for an item at the given index.
     * @param index the zero-relative index of the item
     * @return the color, null if the default text color is used
     * or the index is not valid.
     */
    public Color getEnabledTextColor(int index)
    {
        if (validIndex(index))
        {
           ListItem tempItem = ((ListItem)items.elementAt(index));
           return(tempItem.color);
        }
        return null;
    }

    /**
     * Sets the mode of the DListBox to "MultiColumnListBox
     * mode".  This method always sets the border type to BORDER_NONE,
     * the "ComboBox mode" to be false,
     * and conditionally sets whether the grid should be displayed.
     * @param bCellBorders true to display the cell grid,
     * false to not display the grid
     * @see #BORDER_NONE
     */
    public synchronized void setMultiColumnMode(boolean bCellBorders)
    {
      if(isCellBorders() != bCellBorders)
      {
        setComboMode(false);
        setBorderType(BORDER_NONE);
        setCellBorders(bCellBorders);
        yAdj = 2;
      }
    }

    /**
     * Initializes the list with the string array.
     * @param items the new list items
     */
    public void setListItems(String[] items)
    {
      clear(items.length);   // modifi pour donner la taille
      for (int i = 0; i < items.length; ++i)
         addItem(items[i]);
    }

    /**
     * Returns the current list as an array.
     * @return String[] current list
     * @see #setListItems
     */
    public String[] getListItems()
    {
        int len = countItems();
        String[] items = new String[len];

        for (int i = 0; i < len; ++i)
           items[i] = getText(i);

        return items;
    }

    /**
     * Set a flag indicating that the item at the given zero-relative index
     * has been edited.
     * @param index the zero-relative position of the item
     * @param bCond the "edited" flag
     * @see #isEdited
     */
    public void setEdited(int index, boolean bCond)
    {
      if (validIndex(index))
       {
         ListItem tempItem = ((ListItem)items.elementAt(index));
         tempItem.bEdited = bCond;
       }
    }

    /**
     * Gets the "edited" flag state for the item at the given
     * zero-relative index.
     * @param index the zero-relative position of the item
     * @return true if flagged as edited, false if not
     * @see #setEdited
     */
    public boolean isEdited(int index)
    {
        if (validIndex(index))
        {
            ListItem tempItem = ((ListItem)items.elementAt(index));
            return tempItem.bEdited;
        }

        return false;
    }

    /**
     * Sets whether this list should allow multiple selections or not.
     * @param cond whether to allow multiple selections
     * @see #allowsMultipleSelections
     */
    public void setMultipleSelections(boolean cond)
    {
       if (bMultipleSelections != cond)
        {
         // deselect first because deselect works differently according to bMultipleSelections flag
         if (!cond) deselectAll();
         bMultipleSelections = cond;
        }
    }

    /**
     * Returns true if this list allows multiple selections.
     * @return true if multiple selections allowed, false otherwise
     * @see #setMultipleSelections
     */
    public boolean isMultipleSelections() { return bMultipleSelections; }

    /**
     * Conditionally enables the specified item in the list.
     * @param index the zero-relative position of the item
     * @param cond if true, enable the item; if false, disable the item
     * @see #isEnabled
     */
    public void setEnabled(int index, boolean cond)
    {
      if (validIndex(index))
      {
        ListItem tempItem = ((ListItem)items.elementAt(index));
        if(tempItem.bEnabled != cond)
        {
	     tempItem.bEnabled = cond;
	     if(!cond) deselect(index);
	     tempItem.bDirty = true;
	     if (!bInternalBlockPaint) repaint();
	}
       }
    }

    /**
     * Gets the enabled state of a specific item in the list.
     * @param index the zero-relative position of the item in the list
     * @return true if the item is enabled, false if the item is disabled.
     * @see #setEnabled(int, boolean)
     */
    public boolean isEnabled(int index)
    {
        if (validIndex(index))
        {
            ListItem tempItem = ((ListItem)items.elementAt(index));
            return tempItem.bEnabled;
        }

        return false;
    }

    /**
     * Changes the text associated with the item at the specified
     * zero-relative index.
     * @param index the zero-relative position of the item
     * @param text the new text to associate with the item
     * @see #getText
     */
    public void setText(int index, String text)
    {
        if (validIndex(index))
        {
            ListItem tempItem = ((ListItem)items.elementAt(index));
            tempItem.sText = text;
            tempItem.bDirty = true;
            tempItem.updateWidth(fm);
            updateWidth(tempItem);
            //if (!bInternalBlockPaint) repaint();
        }
    }

    /**
     * Gets the item text associated with the specified zero-relative index.
     * @param index the zero-relative position of the item
     * @return the item text, or null if the index is invalid
     */
    public String getText(int index)
    {
        if (validIndex(index))
            return ((ListItem)items.elementAt(index)).sText;
        else
            return null;
    }

    /**
     * Sets the URL of the image to display at the given zero based index
     * @param aURL the image URL
     * @exception java.net.MalformedURLException
     * if the URL parameter is bad
     * @see #getImage
	 */
   public void setImageURL(int index, URL aUrl) throws java.net.MalformedURLException
   {
	if (validIndex(index))
	{
           ListItem tempItem = ((ListItem)items.elementAt(index));
      	   Image image = getToolkit().getImage(aUrl);
    	   if (image != null) setImage(index, image);
    	   tempItem.url = aUrl;
  	}
   }

    /**
     * Gets the URL of the image displayed at the given zero-based index.
     * @param index the zero-relative index of the image to get
     * @return the URL of the image
     * @see #setImage
	 */
    public URL getImageURL(int index)
    {
      if (validIndex(index))
            return ((ListItem)items.elementAt(index)).url;
      return null;
    }

    /**
     * Change the image associated with an item at a
     * specific zero-relative index.
     * @param index the zero-relative position of the item
     * @param image the image to associate with an item
     * @see #getImage(int)
     */
    public void setImage(int index, Image image)
    {
        if (validIndex(index))
        {
            ListItem tempItem = ((ListItem)items.elementAt(index));

            tempItem.image = image;
            tempItem.bDirty = true;
            if (!bInternalBlockPaint) repaint();
        }
    }

    /**
     * Gets the image associated with the specified zero-relative index.
     * @param index the zero-relative position of the item
     * @return the image at the index, or null if the index is invalid
     */
    public Image getImage(int index)
    {
      if (validIndex(index))
            return ((ListItem)items.elementAt(index)).image;
      return null;
    }

    /**
     * Sets the selection state of the first item in the list which has exactly
     * matching text.
     * @param str the String to select in the list
     */
    public synchronized void setSelectedItem(String str, boolean isSelected)
    {
        ListItem li;
        int z = items.size();
        for (int i = 0; i < z; i++)
        {
          li = (ListItem) items.elementAt(i);

          if(li.sText.equalsIgnoreCase(str))
           if(li.bEnabled)
            {
              setSelectedIndex(i, isSelected);
              return;
            }
        }
    }

    /**
     * Returns the currently selected item in the list.
     * If multiple items are selected, the item most recently
     * selected is returned.
     */
    public synchronized String getSelectedItem()
    {
        return (lastSelected < 0) ? null : getText(lastSelected);
    }

    // for updating
    public void setLastSelected(int index) { lastSelected = index; }
    // lastindex is given by mouse

    /**
     * Select the item at the specified index.
     * This deselects all others when multiple selections disabled.
     * Cette fonction quand on clique sur un item sans CTRL press
     * @param index the zero-relative position of the item to select
     */
    public synchronized void setSelectedIndex(int index, boolean isSelected)
    {
        ListItem listItem = null;

        // multiple selection, on select/dselect an unique element
        if (bMultipleSelections)
        {
          listItem = (ListItem)items.elementAt(index);

          clearSelected();
          listItem.bSelected = isSelected;
          listItem.bDirty = true;
	  if(isSelected) countSelected++;
        }
        else // multiple disabled, on peut dselecter un seul lment
        {
          int s = items.size();

          if(index < s && index > -1)
           {
             listItem = (ListItem)items.elementAt(index);
             boolean wasSelected = listItem.bSelected;

             //If they want to select something, we must deselect everything else.
             if(isSelected) // On veut slectionner
               {
                clearSelected();
                listItem.bSelected = true;
                if(wasSelected)	listItem.bDirty = false;
                countSelected = 1;
               }
             else // On veut d-slectionner
              {
   	        listItem.bSelected = false;
                listItem.tagged = false;
	      	if(wasSelected)
	           {
	       	    listItem.bDirty = true;
          	    countSelected--;
          	   }
              }
        }
      }

      lastSelected = index;
      lastIndex = index;   // DGS rajout
      setVisibleIndex(index);
      if (!bInternalBlockPaint) repaint();
    }

    /**
        DGS
        deselect sans tenir compte des multiple sel ou non
        n'efface pas les tag
    */
    public synchronized void setDeselected(int index)
    {
        if(!validIndex(index)) return;

        ListItem listItem = (ListItem) items.elementAt(index);

    	if(listItem.bSelected)  // Etait slectionn
	     {
          listItem.bSelected = false;
	   	  listItem.bDirty = true;
       	  countSelected--;
       	 }

        lastSelected = -1;
    }

    /** DGS
        Deselect dernier
    */
    public synchronized void setDeselected()
    {
      setDeselected(lastIndex);
      repaint();
    }

    /** DGS modifi
     * Gets the zero-relative index of the selected item in the list.
     * If multiple items are selected, the index of the item most recently
     * selected is returned.
       Modifi pour prendre la liste tagge quand le composant est cach
     * @return the zero-relative index of the selected item, or -1 if no item is
     * selected
     * @see #select, @see #deselect, @see #isSelected
     */
    public synchronized int getSelectedIndex()  { return lastSelected; }

    /**
     * Returns the indexes of all the selected items in the list.
     * @return an array of the zero-relative indexes
     */
    public synchronized int[] getSelectedIndexes()
    {
        int sel[] = new int[countSelected];

        if (countSelected == 1)      sel[0] = lastSelected;
        else if (countSelected > 1)
        {
            int x = 0;
            int s = items.size();
            for (int i = 0; i < s; i++)
            {
                ListItem tempItem = (ListItem)items.elementAt(i);
                if (tempItem.bSelected || tempItem.tagged)
                {
                    sel[x++] = i;
                    if (x == countSelected) break;
                }
            }
        }

        return sel;
    }

    /** DGS modifi
     * Returns all of the the selected items in the list.
       Modifi pour prendre la liste tagge quand le composant est cach
     * @return an array
     * @see #getSelectedItem
     * @see #setSelectedItem
     * @see #isSelected
     */
    public synchronized String[] getSelectedItems()
    {
        String str[] = new String[countSelected];
        ListItem tempItem;

        if (!bMultipleSelections)
        {
            if (lastSelected != -1)
            {
                tempItem = (ListItem)items.elementAt(lastSelected);
                str[0] = tempItem.sText;
            }
        }
        else if (countSelected > 0)
        {
            int x = 0;
            int s = items.size();
            //Disp.it("count selected = " + countSelected + " items size=" + s);
            for (int i = 0; i < s; i++)
            {
                tempItem = (ListItem)items.elementAt(i);
                if (tempItem.bSelected || tempItem.tagged)
                {
                    str[x] = tempItem.sText;
                    //Disp.it( x + ") " +str[x]);
                    if (++x == countSelected)  break;
                }
            }
        }


        return str;
    }

    /**
     * Returns the selected items in the list.
     * Needed in order to implement the java.awt.ItemSelectable interface
     * @return an array of Objects.
     */
    public Object[] getSelectedObjects() { return getSelectedItems();}

    /**
     * Get the selected state of the item at the given index.
     * @param index the zero-relative index of the item to be checked
     * @return true if the item at the specified index
     * is selected; false if it is not selected
     */
    public synchronized boolean isSelected(int index)
    {
      if (validIndex(index))
       {
         ListItem listItem = (ListItem)items.elementAt(index);
         return listItem.bSelected || listItem.deleted;
       }
     return false;
    }



    /**
     * Sets the current list box label.
     * @param the new label
     * @see #getLabel
     */
    public synchronized void setLabel(String label) { ilbLabel = label; }

    /**
     * Returns the current list box label.
     * @return the label
     * @see #setLabel
     */
    public synchronized String getLabel() { return new String(ilbLabel); }

    /**
     * Forces the item at the specified index to be visible in the window.
     * @param index the zero-relative position of the item
     * modified DGS
     */
    public synchronized void setVisibleIndex(int index)
    {
      if(index < nTopRow)
      {
        scrollVertical(index, true);
        return;
      }
      if(index < (nTopRow + visibleRows)) return;

      scrollVertical(index - visibleRows + 1, true);
    }


    /**
     * Forces the item at the specified index to be visible at
     * the top of the window.
     */
    public void setTopRow(int index)
    {

        if (index < 0) return;

        int s = items.size();
        if (s < visibleRows)  index = 0;  // Si total affich
        else
          if(index > (s - visibleRows))  index = s - visibleRows;

        if (nTopRow != index)
        {
          nTopRow = index;
          bAllDirty = true;
          invalidate();
        }
    }

    /**
     * Returns the index of the first item visible in list
     */
    public synchronized int getTopRow() { return nTopRow; }

    /**
     * Sets the number of columns to be used by getMinimumSize().
     * @param columns the number of columns
     * @see #getColumns
     */
    public synchronized void setColumns(int columns)
    {
        if (colsToShow != columns)
        {
            colsToShow = columns;
            invalidate();
        }
    }

    /**
     * Returns the number of columns to be used by minimumSize().
     * @return the number of columns
     * @see #setColumns
     */
    public synchronized int getColumns() { return colsToShow; }

    /**
     * Returns the number of currently visible lines in this list.
     * @return the number of visible lines in the list.
     */
    public int getRows()   {  return visibleRows; }

    /**
     * Set the font for the items in the list box.
     * @param f new font to use for list box items
     */
    public synchronized void setFont(Font f)
    {
	    super.setFont(f);
	    fm = getFontMetrics(f);
	    updateWidths(fm);
	    xCoord = 0;

	    bAllDirty = true;
	    invalidate();
    }

    /**
     * Set the list to be "dirty" (true) to force a complete
     * paint on next repaint, or "clean" (false) to avoid a complete
     * paint on the next repaint.
     * @param isDirty true if the list is dirty (needs repaint)
     * @see #isDirty
     */
    public synchronized void setDirty(boolean isDirty)
      { bAllDirty = isDirty;}

    /**
     * Query the lists all "dirty" state
     * @return true if on the next repaint the entire list will be
     * repainted, or false if the list will not be repainted.
     * @see #setDirty(boolean)
     */
    public synchronized boolean isDirty() { return bAllDirty; }

    /**
     * Query if all rows are selected.
     * @return returns true if all rows in the DListBox are selected
     */
    public boolean isAllSelected()
     { return (countSelected == items.size()); }



    /**
     * Adds an ListItem to the end of the list.
     * @param li the item to add
     */
    protected void addItem(ListItem li)
    {
      if(li != null)
      {
        items.addElement(li);
        updateWidth(li);
        if (!bInternalBlockPaint) repaint();
      }
    }


    // Add sur tableau donn
    // Ajoute avec une image unique

    public synchronized void addItem(int no)
    {
       addItem(new ListItem(image, arrayModel[no], true, fm, bCellBorders));
    }

    // Ajoute avec une image nouvelle

    public synchronized void addItem(Image img, int no)
    {
       addItem(new ListItem(img, arrayModel[no], true, fm, bCellBorders));
    }


    /**
     * Adds an item to the end of the list and enables the item.
     * @param item the item to be added
     */
    public synchronized void addItem(String item)
    {
        addItem(new ListItem((Image)null, item, true, fm, bCellBorders));
    }

    /**
     * Adds an item to the end of the list and conditionally enables it.
     * @param item the item to be added
     * @param bEnabled if true, enable the item; if false, disable
     * the item
     */
    public synchronized void addItem(String item, boolean bEnabled)
    {
        addItem(new ListItem((Image)null, item, bEnabled, fm, bCellBorders));
    }

    /**
     * Adds an item with an image to the end of the list and enables it.
     * @param image the image to display on the item line
     * @param item the item to be added
     */
    public synchronized void addItem(Image image, String item)
    {
        addItem(new ListItem(image, item, true, fm, bCellBorders));
    }

    /**
     * Adds an item with an image to the end of the list and
     * conditionally enables it.
     * @param image the image to display on item line
     * @param item the item to be added
     * @param bEnabled if true, enable the item; if false, disable the item
     */
    public synchronized void addItem(Image image, String item, boolean bEnabled)
    {
        addItem(new ListItem(image, item, bEnabled, fm, bCellBorders));
    }

    /**
     * Adds an item with an image to the end of the list and
     * conditionally enables it. The item text will be displayed in the
     * given color.
     * @param image the image to display on item line
     * @param item the item to be added
     * @param bEnabled if true, enable the item; if false, disable the item
     * @param color the color of the item's text
     */
    public synchronized void addItem(Image image, String item, boolean bEnabled, Color color)
    {
        ListItem li = new ListItem(image, item, bEnabled, fm, bCellBorders);
        li.color = color;
        addItem(li);
    }



    /**
     * Inserts an item with an image at a specific zero-relative index and
     * conditionally enables it.
     * @param index the zero-relative index to insert at
     * @param image the image to display on the item line
     * @param item the item to be added
     * @param bEnabled if true, enable the item; if false, disable the item
     */
    public synchronized void insertItem(int index, Image image, String item, boolean bEnabled)
    {
        ListItem li = null;
        if ( validIndex(index) )
        {
            items.insertElementAt(li = new ListItem(image, item, bEnabled, fm, bCellBorders), index);
            bAllDirty = true;
            if (lastSelected >= index)
                lastSelected++;
        }
        else
            items.addElement(li = new ListItem(image, item, bEnabled, fm, bCellBorders));
        updateWidth(li);

        if (!bInternalBlockPaint) repaint();
    }

    /**
     * Inserts a vector list of items at a specific zero-relative index and
     * conditionally enables them.
     * Each item will be displayed with the given image.
     * @param index the zero-relative index to insert at
     * @param itemVector the item vector to be added
     * @param image the image to display for all inserted items
     * @param bEnabled if true, enable the items; if false, disable the items
     */
    public synchronized void insertItems(int index, Vector itemVector, Image image, boolean bEnabled)
    {
        int s = itemVector.size();
        int x = 0;

        if ( validIndex(index) )
        {
            for (x = 0; x < s; x++)
                items.insertElementAt(new ListItem(image, (String) itemVector.elementAt(x), bEnabled, fm, bCellBorders), index++);
        }
        else
        {
            for (x = 0; x < s; x++)
                items.addElement(new ListItem(image, (String) itemVector.elementAt(x), bEnabled, fm, bCellBorders));
        }
        deselectAll();
        bAllDirty = true;
        updateWidths(fm);

        if (!bInternalBlockPaint) repaint();
    }

    /**
     * Returns the total number of items in the list.
     * @return the number of items in the list
     */
    public int countItems() { return items.size(); }

   /**
     * Clears the list.
       DGS
     */

    public synchronized void clear() { this.clear(16); }

    public synchronized void clear(int capacity)
    {
        items = new Vector(capacity, 16);

        nTopRow = 0;
        //visibleIndex = -1;
        lastDownModifiers = -1;
        lastSelected = -1;
        lastIndex = -1;
        lastTempIndex = -1;
        countSelected = 0;
        prevSelectTime = -1;
        prevSelectRow = -1;
        VBar.setValues(1, 1, 0, 2);
        VBar.setVisible(false);
        bVBarVisible = false;
        HBar.setValues(1, 1, 0, 2);
        HBar.setVisible(false);
        bHBarVisible = false;
        bAllDirty = true;
        bInternalBlockPaint = false;
        bBlockPaint = false;
        longestLineValue = 0;
        xCoord = 0;
        //repaint();
    }



    /**
     * Delete an item from the list.
     * @param index the zero-relative index of the item to delete
     * @see #delItems
     * @see #delSelectedItems
       modifi DGS
     */
    public synchronized void delItem(int index)
    {
        if (validIndex(index))
        {
            items.removeElementAt(index);
            bAllDirty = true;
            scrollVertical(nTopRow, true);
            updateWidths(null);
            if (!bInternalBlockPaint) repaint();
        }
    }

    /**
     * Delete multiple items from the list.
     * Note that the end index must be greater than or
     * equal to the start index.
     * @param start the zero-relative start index of the items
     * @param end the zero-relative end index of the items
     * @see #delItem
     * @see #delSelectedItems
     */
    public synchronized void delItems(int start, int end)
    {
        int s = items.size();
        if (s > 0)
        {
            if (end >= s)   end = s - 1;
            if (start < 0)  start = 0;
            if (start <= end)
            {
                bInternalBlockPaint = true;
                for (int i = end; i >= start; i--)
                {
				//	setSelectedIndex(i, false);
	                items.removeElementAt(i);
                }
                bInternalBlockPaint = false;
                bAllDirty = true;
                scrollVertical(nTopRow, true);
                updateWidths(null);
                repaint();

            }
        }
    }

    /**
     * Deletes the currently selected items from the list.
     * @see #delItem
     * @see #delItems
     */
    public synchronized void delSelectedItems()
    {
        int s = items.size();

        bInternalBlockPaint = true;
        for (int x = 0; x < s; )
        {
            ListItem tempItem = (ListItem) items.elementAt(x);
            if (tempItem.bSelected)
            {
                //setSelectedIndex(x, false);
                delItem(x);
                s--;
            }
            else
                x++;
        }

        bAllDirty = true;
        scrollVertical(nTopRow, true);
        updateWidths(null);
        bInternalBlockPaint = false;
        repaint();
    }

    /**
     * Selects an item at the specified index in a multiple-selection-enabled DListBox
     * using Shift, Ctrl or Ctrl-Shift modifiers to mimic mouse
     * selecting with the Shift, Ctrl or Ctrl-Shift modifiers.
     * @param index the zero-relative position of the item to select
     * @param bShift whether to select with the Shift modifier
     * @param bControl whether to select with the Ctrl modifier
     */
    public synchronized void selectMultiple(int index, boolean bShift, boolean bControl)
    {
        if (!bMultipleSelections)
        {
            bShift = false;
            bControl = false;
        }
        if (bShift)        shiftSelect(index, bControl);
        else if (bControl) ctrlSelect(index);
        else setSelectedIndex(index, true);
    }

    /**
     * Selects all the items in the list.
     * @see #select
     * @see #deselectAll
     */
    public synchronized void selectAll()
    {
        if (!bMultipleSelections)  return;

        int s = items.size();
        ListItem li;
        for (int i = 0; i < s; i++)
        {
            li = (ListItem)items.elementAt(i);
            if (li.bEnabled) li.bSelected = true;
        }
        countSelected = s;
        lastSelected = s - 1;
        repaint();
    }

    /**
     * deprecated
     */

    public synchronized void deselect(int index)
    {
        if (validIndex(index))
        {
			setSelectedIndex(index, false);
            if (!bInternalBlockPaint) repaint();
        }
    }

    /**
     * Deselects all items in the list.
     * @see #selectAll
     */
    public synchronized void deselectAll()
    {
        ListItem li;
        if (!bMultipleSelections)
        {
         if (lastSelected != -1)  // Le dernier est le seul
          {
           li = (ListItem)items.elementAt(lastSelected);
           li.bSelected = false;
           li.bDirty = true;
           li.deleted = false;   // Enlve aussi si barr
           li.tagged = false;
          }
        }
        else
        {
           int s = items.size();
           for (int i = 0; i < s; i++)
            {
              li = (ListItem)items.elementAt(i);
              li.bSelected = false;
              li.bDirty = true;
              li.deleted = false;
              li.tagged = false;
            }
        }
        lastSelected = -1;
        countSelected = 0;
        if(!bInternalBlockPaint) repaint();
    }


    /** Enlve tous les barrs
        Dja dans deselectAll
    */
    public synchronized void uncrossAll()
    {
     int m = items.size();
     for(int i = m; i >= 0; i--) crossOut(i, false);
    }

    /**
       Comme deselectAll sans repaint et plus simple
       ne modifie pas les barrs
       DGS
      */

    public synchronized void clearSelected()
    {
      ListItem li;
      int s = items.size();

      for (int i = 0; i < s; i++)
       {
        li = (ListItem) items.elementAt(i);
        if(li.bSelected || li.tagged)
        {
         li.bSelected = false;
         li.tagged = false;
         li.bDirty    = true;
        }
       }

     lastSelected = -1;
     countSelected = 0;
    }


    /**
     * Returns the preferred dimensions needed for the list with
     * the specified number of rows.
     * @param rows the number of visible rows in the list
     * @return the preferred size to show the specified number of rows
     */
    public Dimension getPreferredSize(int rows)
    {
        Dimension d = getMinimumSize(rows);
        Dimension s = getSize();
        return new Dimension(Math.max(d.width, s.width), Math.max(d.height, s.height));
    }

    /**
	 * Returns the recommended dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the recommended size of this component.
     *
     * The size returned is big enough to show the maximum number of visible
     * rows as requested when constructed, or all the rows currently in the list.
     *
     * @see #minimumSize
	 */
    public Dimension getPreferredSize()
    {
        if (rowsToShow > 0)  return getPreferredSize(rowsToShow);
        return getPreferredSize(items.size());
    }

    /**
     * Returns the minimum dimensions needed to show the given number of rows
     * in the list.
     * @param rows the number of visible rows in the list
     * @return the minimum size of this list to show the given number of rows
     */
    public Dimension getMinimumSize(int rows)
    {
        font = getFont();
        if (font != null)
        {
            fm = getFontMetrics(font);
            if (fm != null)
            {
                fontHeight = fm.getHeight();

                if (bCellBorders)
                    cellHt = fontHeight+5;
                else
                    cellHt = fontHeight+1;

                return new Dimension( ((fm.stringWidth("WN") * colsToShow) / 2) + LINE_SLOP, (rows * cellHt + borderWidth + yAdj) );
            }
        }
        return new Dimension(borderWidth + LINE_SLOP, borderWidth + yAdj);
    }

    /**
	 * Returns the minimum dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the minimum size of this component.
     *
     * The size returned is big enough to show the maximum number of visible
     * rows as requested when constructed, or all the rows currently in the list.
     *
     * @see #preferredSize
	 */
    public Dimension getMinimumSize()
    {
        if (rowsToShow > 0)
            return getMinimumSize(rowsToShow);
        return getMinimumSize(items.size());
    }

	/**
	 * Tells this component that it has been added to a container.
	 * This is a standard Java AWT method which gets called by the AWT when
	 * this component is added to a container. Typically, it is used to
	 * create this component's peer.
	 *
	 * It has been overridden here to hook-up event listeners.
     * It is also used to handle list metrics after the peer is created.
	 *
	 * @see #removeNotify
	 */
  public synchronized void addNotify()
  {
	super.addNotify();

	if (mouseMotion == null)
	{
		mouseMotion = new MouseMotion();
		addMouseMotionListener(mouseMotion);
	}
        VBar.addAdjustmentListener(this);
        HBar.addAdjustmentListener(this);

	if (focus == null)
	{
		focus = new Focus();
		addFocusListener(focus);
	}

         font = getFont();
         fm = getFontMetrics(font);
         fontHeight = fm.getHeight();
         updateWidths(fm);
	}

	/**
	 * Tells this component that it is being removed from a container.
	 * This is a standard Java AWT method which gets called by the AWT when
	 * this component is removed from a container. Typically, it is used to
	 * destroy the peers of this component and all its subcomponents.
	 *
	 * It has been overridden here to unhook event listeners.
	 *
	 * @see #addNotify
	 */
  public synchronized void removeNotify()
  {
	if (mouseMotion != null)
	{
		removeMouseMotionListener(mouseMotion);
		mouseMotion = null;
	}
	removeKeyListener(this);
        VBar.removeAdjustmentListener(this);
        HBar.removeAdjustmentListener(this);


        if (focus != null)
       	{
		removeFocusListener(focus);
		focus = null;
	}

		super.removeNotify();
   }

    /**
     * Blocks the repainting of the control.
     * @param cond true to prevent updating the component,
     * false for normal painting behavior
     */
    public synchronized void blockPaint(boolean cond)
    {
        bBlockPaint = cond;
        if (!bBlockPaint)  repaint();
    }

    // bloque temporairement paint
    public synchronized void enablePainting(boolean m)
    {  bInternalBlockPaint = !m; }

    /**
     * Scrolls the DListBox vertically.
     * @param info either an absolute value or Event.SCROLL_PAGE_DOWN,
     * Event.SCROLL_PAGE_UP, Event.SCROLL_LINE_DOWN, Event.SCROLL_LINE_UP.
     * The range for an absolute value should be between 0 and (number of rows-1).
     * @param bAbsolute true to treat the info parameter as an absolute value,
     * false to treat it as one of the SCROLL_... constants
     */
    public synchronized void scrollVertical(int info, boolean bAbsolute)
    {
        int temp = nTopRow;

        if (visibleRows == 0) temp = 0;
        else
        {
            if (bAbsolute) temp = info;
            else
            {
                switch (info)
                {
                    case AdjustmentEvent.BLOCK_DECREMENT:
                         temp -= visibleRows; break;
                    case AdjustmentEvent.BLOCK_INCREMENT:
                         temp += visibleRows; break;
                    case AdjustmentEvent.UNIT_DECREMENT:
                         temp -= 1; break;
                    case AdjustmentEvent.UNIT_INCREMENT:
                         temp += 1; break;
                }
            }

            if (temp < 0)  temp = 0;
            else if (temp > (items.size() - visibleRows))
            {
                temp = items.size() - visibleRows;
                if (temp < 0)  temp = 0;
            }
        }

        if (nTopRow != temp)
        {
            nTopRow = temp;
            VBar.setValue(temp);
            bAllDirty = true;
            if (!bInternalBlockPaint)
               repaint();
        }
    }

    /**
     * Scrolls the DListBox horizontally
     * @param info either an absolute value or Event.SCROLL_PAGE_DOWN,
     * Event.SCROLL_PAGE_UP, Event.SCROLL_LINE_DOWN, Event.SCROLL_LINE_UP.
     * The range for absolute value should be between 0 and the width of the
     * largest item, in pixels
     * @param bAbsolute true to treat the info parameter as an absolute value,
     * false to treat it as one of the SCROLL_... constants
     */
    public synchronized void scrollHorizontal(int info, boolean bAbsolute)
    {
        int temp = xCoord;

        if (bAbsolute) temp = -info;
        else
        {
            switch (info)
            {
                case AdjustmentEvent.BLOCK_DECREMENT:
                     temp += lWidth; break;
                case AdjustmentEvent.BLOCK_INCREMENT:
                     temp -= lWidth; break;
                case AdjustmentEvent.UNIT_DECREMENT:
                     temp += 1; break;
                case AdjustmentEvent.UNIT_INCREMENT:
                     temp -= 1; break;
            }
        }

        if (temp > 0)  temp = 0;
        else if ( (-temp) > HBar.getMaximum() - lWidth)
            temp = -(HBar.getMaximum() - lWidth);

        if (xCoord != temp)
        {
            xCoord = temp;
            HBar.setValue(-temp);
            bAllDirty = true;
            if (!bInternalBlockPaint) repaint();
        }
    }

    /**
     * Handles redrawing of this component on the screen.
     */
    public synchronized void update(Graphics g)
    {
        if (!bBlockPaint && !bInternalBlockPaint)  paint(g);
    }

   /**
     * Paints this component using the given graphics context.
     */
    public synchronized void paint(Graphics g)
    {
        ListItem item;

        int nItems;
        int xSelectStartLoc;
        int i, j, rows;
        int vWid = 0;
        int hHt = 0;
        int gridAdj = 0;
        int yLoc = 0;
        int ySLoc = 0;
        boolean bShowV = false;
        boolean bShowH = false;

	// are cached colors correct?

	Color curBackground = getBackground();
	if (!Tool.objectsEqual(curBackground , cachedBackground))
	{
	  cachedBackground = curBackground;
	  calculateBorderColors(PanelColor);
	}

        // Set size of ListBox

        Rectangle rect = getBounds();
        font = g.getFont();
        fm = g.getFontMetrics(font);
        fontHeight = fm.getHeight();
        fontDescent = fm.getDescent();

        nItems = items.size();

        if (bCellBorders) cellHt = fontHeight+5;
        else              cellHt = fontHeight+1;

        if (nItems == 0)
        {
            nTopRow = 0;
            visibleRows = 0;
            bShowH = false;
            bShowV = false;
            xCoord = 0;
        }
        else
        {
            if ( bAllowShowHBar && ( longestLineValue > (rect.width - borderWidth)) )
            {
                bShowH = true;
                hHt = barSize;
            }
            else
            {
                bShowH = false;
                hHt = 0;
                xCoord = 0;   // Rajout: pas d'asc hor, pas de dplacement
            }

            rows = (rect.height - hHt - borderWidth - yAdj) / cellHt;
            //rows = (rect.height - hHt - borderWidth - 4) / cellHt;

            if ( bAllowShowVBar && ( nItems > rows ))
            {
                bShowV = true;
                vWid = barSize;
                if (!bShowH)
                {
                    if ( bAllowShowHBar && (longestLineValue > (rect.width - borderWidth - vWid)) )
                    {
                        bShowH = true;
                        hHt = barSize;
                        rows = (rect.height - hHt - borderWidth - yAdj) / cellHt;
                        //rows = (rect.height - hHt - borderWidth - 4) / cellHt;
                    }
                }
            }
            else
            {
                bShowV = false;
                vWid = 0;
            }

            if (visibleRows != rows)
            {
                visibleRows = rows;
                bAllDirty = true;
            }

            if (bShowV)
            {
                VBar.setBounds(rect.width - barSize - halfBorderWidth, halfBorderWidth, barSize, rect.height - borderWidth - hHt);
                VBar.setValues(nTopRow, visibleRows, 0, nItems);
                VBar.setBlockIncrement(visibleRows);
                lWidth = rect.width - vWid - borderWidth;

                if (!bVBarVisible)
                {
                    bVBarVisible = true;
                    VBar.setVisible(true);
                }
            }
            else
            {
                lWidth = rect.width - borderWidth;

                if (bVBarVisible)
                {
                    bVBarVisible = false;
                    VBar.setVisible(false);
                }
            }

            if (bShowH)
            {
                HBar.setBounds(halfBorderWidth, rect.height-barSize-halfBorderWidth, rect.width-borderWidth-vWid, barSize);
                HBar.setValues(-xCoord, lWidth, 0, longestLineValue);
                HBar.setBlockIncrement(lWidth);

                if (!bHBarVisible)
                {
                    bHBarVisible = true;
                    HBar.setVisible(true);
                }
            }
            else
            {
                if (bHBarVisible)
                {
                    bHBarVisible = false;
                    HBar.setVisible(false);
                }
            }
        }

        if (nItems == 0 || bAllDirty)
        {
            g.clearRect(halfBorderWidth, halfBorderWidth, rect.width - borderWidth - 1, rect.height - borderWidth - 1);
	}

        if (borderType == BORDER_REGULAR)
        {
            g.setColor(borderDarkerColor);
            g.drawLine(0, 0, rect.width - 1, 0);
            g.drawLine(0, 0, 0, rect.height - 1);

            g.setColor(borderDarkColor);
            g.drawLine(1, 1, rect.width - 2, 1);
            g.drawLine(1, 1, 1, rect.height - 2);

            g.setColor(borderLightColor);
            g.drawLine(1, rect.height - 2, rect.width - 2, rect.height - 2 );
            g.drawLine(rect.width - 2, 1, rect.width - 2, rect.height - 2 );

            g.setColor(borderLighterColor);
            g.drawLine(0, rect.height - 1, rect.width - 1, rect.height - 1 );
            g.drawLine(rect.width - 1, 0, rect.width - 1, rect.height - 1 );
        }

        g.clipRect(halfBorderWidth, halfBorderWidth, lWidth, rect.height - borderWidth);

        if (nTopRow >= nItems)  nTopRow -= visibleRows + 1;

        i = nTopRow;
        j = Math.min(nTopRow + visibleRows, nItems);

        // Boucle d'affichage
        for (int dp = 0; i < j; ++i, ++dp)
        {
            yLoc = (dp * cellHt) + yAdj + halfBorderWidth;
            ySLoc = ( (dp + 1) * cellHt) + yAdj + halfBorderWidth;

            item = (ListItem) items.elementAt(i);
            if (item.bDirty || bAllDirty)
            {
                item.bDirty = false;
                g.clearRect(halfBorderWidth, yLoc, lWidth, cellHt);
            }

            if (item.image != null)
                xSelectStartLoc = halfBorderWidth + IMAGE_WIDTH + 2;
            else
                xSelectStartLoc = halfBorderWidth;

            if (item.bCellBorder) gridAdj = 2;
            else                  gridAdj = 0;


            int tWidth = lWidth - xCoord - IMAGE_WIDTH - 2;
            if (item.bSelected)
            {
               g.setColor(textHighlight);
               g.fillRect(xCoord + xSelectStartLoc + gridAdj, yLoc, tWidth, cellHt);
               g.setColor(textHighlightText);
            }
            else
            {
              // Color of text, global or that of the row
                if (item.bEnabled)
                {
                  if (item.color == null) g.setColor(enabledColor);
                  else                    g.setColor(item.color);
                }
                else g.setColor(disabledColor);
            }
            // Ligne de texte
            g.drawString(item.sText, xCoord + xSelectStartLoc + gridAdj + 2, ySLoc-fontDescent);
            // Image ventuelle
            if (item.image != null)
                g.drawImage(item.image, xCoord + halfBorderWidth + gridAdj + 2, yLoc+2, IMAGE_WIDTH, cellHt-3, this);

            // cross out with a red line
            if(item.deleted)
            {
             g.setColor(Color.red);
             int yy = yLoc + cellHt / 2;
             g.drawLine(// xCoord + xSelectStartLoc + gridAdj,
                 xCoord + halfBorderWidth + gridAdj + 2, yy, lWidth, yy);
            }

            if (item.bCellBorder)
            {
                g.setColor(item.cellBorderColor);
                g.drawRect(halfBorderWidth, yLoc, lWidth, cellHt);
            }

            if(item.tagged)
            {
             g.setColor(textHighlight);
             g.drawRect(xCoord + xSelectStartLoc + gridAdj, yLoc + 1, tWidth, cellHt - 2);
            }

        } // boucle dp

        bAllDirty = false;
        listChanged = false;
    }



    /**
     * Moves and/or resizes this component.
     * This is a standard Java AWT method which gets called to move and/or
     * resize this component. Components that are in containers with layout
     * managers should not call this method, but rely on the layout manager
     * instead.
     *
     * @param x horizontal position in the parent's coordinate space
     * @param y vertical position in the parent's coordinate space
     * @param width the new width
     * @param height the new height
     */
    public synchronized void setBounds(int x, int y, int width, int height)
    {
        bAllDirty = true;
        super.setBounds(x, y, width, height);
    }

    //--------------------------------------------------
    // event methods
    //--------------------------------------------------

    /**
     * Adds the specified item listener to receive item events
     * @param l the item listener
     * @see #removeItemListener
     * @see #sourceItemEvent
     */
	public void addItemListener(ItemListener l)
	{
		itemListener = AWTEventMulticaster.add(itemListener, l);
	}

    /**
     * Removes the specified item listener so it no longer receives
     * item events.
     * @param l the action listener
     * @see #addItemListener
     * @see #sourceItemEvent
     */
	public void removeItemListener(ItemListener l)
	{
		itemListener = AWTEventMulticaster.remove(itemListener, l);
	}

    /**
     * Adds the specified action listener to receive action events
     * from this component.
     * @param l the action listener
     */
	public void addActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.add(actionListener, l);
	}

    /**
     * Removes the specified action listener so it no longer receives
     * action events from this component.
     * @param l the action listener
     */
	public void removeActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, l);
	}



	/**
	 * This is the Mouse Event handler
	 */
	 int popx;
	 int popy;

	public void mousePressed(MouseEvent e)
	{
	    requestFocus();

		int x = e.getX();
		int y = e.getY();
		int index = -1;

		if (e.isPopupTrigger())  // right button sur PC
		{
		   popx = e.getX();
		   popy = e.getY();
  	           sendActionEvent("Popup");
 	           return;
                }

            fastDownCount++;
            bMouseDrawHandled = false;

            if ( (x > -1) && (x < lWidth) )
            {
                index = mouseCalcIndex(y);
                if (index == -1) return;

                if (bMultipleSelections)
                {
		  if(e.isShiftDown() && e.isControlDown())
		  {
                     bMouseDrawHandled = true;
		     shiftSelect(index, true);
		   }
		   else if(e.isShiftDown())
		   {
                     bMouseDrawHandled = true;
		     shiftSelect(index, false);
		   }
		   else if(e.isControlDown())
		   {
		      bMouseDrawHandled = true;
		      ctrlSelect(index);
		   }
		   else
		   {
		      setSelectedIndex(index, true);
                   }
                }
                else
                {
                    lastTempIndex = index;
		     setSelectedIndex(index, true);
                }
                repaint();
            }
		}

		public void mouseReleased(MouseEvent e)
		{

		int x = e.getX();
		int y = e.getY();
		int index = -1;
	        boolean bDoubleClick;
		long selectTime;

    		if (e.isPopupTrigger())  // right button sur PC
		{
		   popx = e.getX();
		   popy = e.getY();
		   sendActionEvent("Popup");
		   requestFocus();
 		   return;
                }


            fastDownCount--;
            if (bMouseDrawHandled) // multi-selected ctrl or shift
            {
                notifyHelper(-1, y, -1, false);
                return;
            }

            index = mouseCalcIndex(y);
            if ( (index == -1) || (x < 0) || (x >= lWidth) )
            {
                fastDownCount = 0;
                index = lastTempIndex;
                if (index == -1) return;
            }

            if (bComboMode)
            {
                fastDownCount = 0;
                bDoubleClick = false;
            }
            else
            {
                // sometimes you get two MOUSE_DOWNS before the first MOUSE_UP
                if (fastDownCount > 0)
                {
                    fastDownCount = 0;
                    bDoubleClick = true;
                    prevSelectTime = -1;
                    prevSelectRow = -1;
                }
                else
                {
                    selectTime = System.currentTimeMillis();
                    if ( (prevSelectTime != -1) &&              // valid prevSelectTime
                         (prevSelectRow == index) &&            //
                         ((prevSelectTime + 250) > selectTime)  // A double click
                      )
                    {
                        bDoubleClick = true;
                        prevSelectTime = -1;
                        prevSelectRow = -1;
                    }
                    else
                    {
                        bDoubleClick = false;
                        prevSelectTime = selectTime;
                        prevSelectRow = index;
                    }
                }
            }
            if (lastSelected != index)
            {
                setSelectedIndex(index, true);
                repaint();
            }
            if (bMultipleSelections)
            {
                notifyHelper(x, y, lastSelected, bDoubleClick);
                lastIndex = lastSelected;
            }
            else
            {
                notifyHelper(x, y, index, bDoubleClick);
                lastIndex = index;
            }
            lastTempIndex = -1;
            lastDownModifiers = -1;
		}

		public void mouseExited(MouseEvent e)
		{
	        if (!bMultipleSelections && bComboMode)
	            lastIndex = -1;
		}

		public void mouseEntered(MouseEvent e) {}
		public void mouseClicked(MouseEvent e) {}

	/**  MouseMotion Event handling innerclass */

	class MouseMotion implements java.awt.event.MouseMotionListener, java.io.Serializable
	{
		/** Handles Mouse Moved events */
		public void mouseMoved(MouseEvent e)
		{
			if(!bMultipleSelections)
			{
				int x = e.getX();
				int y = e.getY();
				int  index = -1;

	            if ( (x > -1) && (x < lWidth) )
	            {
	                if (bComboMode)
	                {
	                    index = mouseCalcIndex(y);
	                    if (index == -1 || index == lastIndex) return;
	                    lastIndex = index;

	                    setSelectedIndex(index, true);
	                    repaint();
	                }
		    }
	  	  }
		}

		/** Handles Mouse Dragged events */
	   public void mouseDragged(MouseEvent e)
	   {
		int x = e.getX();
		int y = e.getY();
		int  index = -1;

            if ( (x > -1) && (x < lWidth) )
            {
                if (bComboMode)
                {
                    index = mouseCalcIndex(y);
                    if (index == -1 || index == lastIndex) return;
                    lastIndex = index;

                    setSelectedIndex(index, true);
                    repaint();
                }
                else
                {
                    index = mouseCalcIndex(y);
                    if (index != -1 && index != lastIndex)
                    {
                        if (bMultipleSelections)
                        {
                            if (e.getModifiers() != 0) return;

	                        bInternalBlockPaint = true;
	                        setVisibleIndex(index);
	                        bInternalBlockPaint = false;
	                        shiftSelect(index, false);
                        }
                        else
                        {
                        	setSelectedIndex(index, true);
						}
                        lastIndex = index;
                    }
                }
            }
		}
	}


  /** Handles Key Pressed events */
  public void keyReleased(KeyEvent e) { }
  public void keyTyped(KeyEvent e) { }
  public void keyPressed(KeyEvent e)
  {
       int code = e.getKeyCode();

       if (isVisible() == false) return;
       if (e.isMetaDown() || e.isAltDown())  return;

       int index = -1;

       // with CTRL
       if(e.isControlDown())
       {
         switch(code)
         {
          case KeyEvent.VK_UP:
	       index = keyCalcIndex(-1, false);
	       break;
          case KeyEvent.VK_DOWN:
	       index = keyCalcIndex(1, false);
               break;
         }
         if (index == -1) return;

         if (bMultipleSelections)
         {
            setVisibleIndex(index);
            ctrlSelect(index);
            repaint();
            notifyHelper(-1, -1, index, false);
            prevSelectTime = -1;
            prevSelectRow = -1;
	 }

         return;
       }  // end ctrm


       // without CTRL
       switch(code)
        {
          case KeyEvent.VK_ENTER:  sendActionEvent("Enter"); requestFocus(); break;
          case KeyEvent.VK_DELETE: sendActionEvent("Delete"); requestFocus(); break;
          case KeyEvent.VK_ESCAPE: sendActionEvent("Esc"); break;
          case KeyEvent.VK_LEFT:   sendActionEvent("Up"); requestFocus(); break;

          case KeyEvent.VK_HOME:
               index = keyCalcIndex(0, true);
               break;
          case KeyEvent.VK_END:
	       index = keyCalcIndex(items.size() - 1, true);
	       break;
          case KeyEvent.VK_PAGE_UP:
	       index = keyCalcIndex(-visibleRows, false);
	       break;
          case KeyEvent.VK_PAGE_DOWN:
	       index = keyCalcIndex(visibleRows, false);
	       break;
          case KeyEvent.VK_UP:
	       index = keyCalcIndex(-1, false);
	       break;
          case KeyEvent.VK_SPACE:
	       index = keyCalcIndex(1, false);
               if (index == -1) return;
               if (bMultipleSelections)
               {
                     setVisibleIndex(index);
                     ctrlSelect(index);
                     repaint();
                     notifyHelper(-1, -1, index, false);
                     prevSelectTime = -1;
                     prevSelectRow = -1;
               }
               return;
          case KeyEvent.VK_DOWN:
	       index = keyCalcIndex(1, false);
               break;

          default:
               if((code >= KeyEvent.VK_A) && (code <= KeyEvent.VK_Z))
               {
                   int li = lastIndex;
                   if(li < 0 || li >= items.size()) li = -1;
                   li++;
                   int i = moveCursor(code, li);
                   if(i > -1) index = keyCalcIndex(i, true);
               }
               break;
         }


          if (index == -1) return;

          if (bMultipleSelections && e.isShiftDown())
           {
            setVisibleIndex(index);
            shiftSelect(index, false);
            return;
	   }

          // regular select, multiple or single
          setSelectedIndex(index, true);
          repaint();
          notifyHelper(-1, -1, index, false);
          prevSelectTime = -1;
          prevSelectRow = -1;

    }

/**
 * Handles ScrollBar events
 * @param e the AdjustmentEvent
 */
    public void adjustmentValueChanged(AdjustmentEvent e)
    {
	//We are only listening to the VBar and the HBar...
	if(e.getSource() == VBar)
	{
          if(e.getAdjustmentType() == AdjustmentEvent.TRACK)
	    scrollVertical(e.getValue(), true);
 	  else
	    scrollVertical(e.getAdjustmentType(), false);
	}
	else if(e.getSource() == HBar)
	{
	   if(e.getAdjustmentType() == AdjustmentEvent.TRACK)
	     scrollHorizontal(e.getValue(), true);
	   else
	     scrollHorizontal(e.getAdjustmentType(), false);
	}
    }

    /**
     * Fire an item event to the listeners
     * @see #addItemListener
     * @see #removeItemListener
     */
    protected void sourceItemEvent()
    {
	if (itemListener != null)
	  itemListener.itemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, this, ItemEvent.SELECTED));
    }


    /**
     * Determines if the given index is valid.
     */
    protected boolean validIndex(int index)
    {
      return (index >= 0) && (index < items.size());
    }

    /**
     * Determines if the item at the given index is visible.
     */
    protected boolean isVisibleIndex(int index)
    {
      return  (index >= nTopRow) && (index < (nTopRow + visibleRows));
    }

   /**
     * Returns a string representing the state of this object.
     * This is a standard Java AWT method which can be usefull for debugging.
     *
     * @return a handy string for debugging purposes
     */
    protected String paramString()
    {
        return super.paramString() + ", selected=" + getSelectedItem();
    }

    /**
     * Selects an index in a multiple-selection-enabled DListBox
     * using Shift or Ctrl-Shift modifiers to mimic mouse
     * selecting with the Shift or Ctrl-Shift modifiers.
     * @param index the zero-relative position of the item to select
     * @param bControl whether to select with the Ctrl modifier
     */
    protected void shiftSelect(int index, boolean bControl)
    {
        int i = 0;
        int s = 0;
        ListItem li;

        if (lastSelected == -1)
            setSelectedIndex(index, true);
        else if (lastSelected == index)
        {
            if (!bControl)
            {
                bInternalBlockPaint = true;
                setSelectedIndex(index, true);
                bInternalBlockPaint = false;
            }
        }
        else if (lastSelected < index)
        {
            if ( !isEnabled(index) )  return;

            if (bControl)
            {
                i = lastSelected + 1;
                s = Math.min(items.size(), index + 1);
            }
            else
            {
                i = 0;
                s = items.size();
            }
            for (; i < s; i++)
            {
                li = (ListItem)items.elementAt(i);
                if (i < lastSelected || i > index)
                {
                    if (li.bSelected)
                    {
                        li.bDirty = true;
                        li.bSelected = false;
                        countSelected--;
                    }
                }
                else
                {
                    if (!li.bSelected)
                    {
                        li.bDirty = true;
                        li.bSelected = true;
                        countSelected++;
                    }
                }
            }
        }
        else if (lastSelected > index)
        {
            if ( !isEnabled(index) )
                return;

            if (bControl)
            {
                i = index;
                s = Math.min(items.size(), lastSelected);
            }
            else
            {
                i = 0;
                s = items.size();
            }
            for (i = 0; i < s; i++)
            {
                li = (ListItem)items.elementAt(i);
                if (i < index || i > lastSelected)
                {
                    if (li.bSelected)
                    {
                        li.bDirty = true;
                        li.bSelected = false;
                        countSelected--;
                    }
                }
                else
                {
                    if (!li.bSelected)
                    {
                        li.bDirty = true;
                        li.bSelected = true;
                        countSelected++;
                    }
                }
            }
        }
        if (!bInternalBlockPaint) repaint();
    }

    /**
     * Selects an index using the Ctrl modifier
     * to mimic mouse selecting with the Ctrl modifier.
     * @param index the zero-relative position of the item to select
     */
    protected void ctrlSelect(int index)
    {
        ListItem listItem = (ListItem)items.elementAt(index);
        if (!listItem.bEnabled) return;
        if (listItem.bSelected)
        {
            listItem.bSelected = false;
            countSelected--;
            lastSelected = -1;
        }
        else
        {
            listItem.bSelected = true;
            countSelected++;
            lastSelected = index;
        }

        listItem.bDirty = true;
        if (!bInternalBlockPaint) repaint();
    }

    /**
     * A utility routine that determines the index of the item the mouse is in,
     * given the vertical coordinate of the mouse.
     * @param y mouse vertical (y) coordinate
     * @return the zero-relative index of the item the mouse is in, or -1 if
     * past the end of the list
     */
    protected int mouseCalcIndex(int y)
    {
        int mci;

        if (y < yAdj)
            mci = nTopRow - 1;
        else
            mci = ((y - yAdj) / cellHt) + nTopRow;

        if (mci >= items.size())
            mci = -1;
        else if (mci < 0)
            mci = 0;

        return mci;
    }


    protected void updateWidth(ListItem li)
    {
     int lineWidth = li.lineWidth + LINE_SLOP;

     if (li.image != null) lineWidth += IMAGE_WIDTH;
     if (lineWidth > longestLineValue)
        longestLineValue = lineWidth;

     listChanged = true;
    }

    protected void updateWidths(FontMetrics fm)
    {
      int s = items.size();
      int lineWidth;
      ListItem li;

      longestLineValue = 0;
      for (int x = 0; x < s; x++)
       {
         li = (ListItem) items.elementAt(x);
         li.updateWidth(fm);
         lineWidth = li.lineWidth + LINE_SLOP;
         if (li.image != null) lineWidth += IMAGE_WIDTH;
         if (lineWidth > longestLineValue) longestLineValue = lineWidth;

       }
      listChanged = true;
    }

	/**
	 * Utility routine to generate action and/or item events in response to
	 * key or mouse events.
	 * @param x mouse component-relative horizontal position, in pixels
	 * @param x mouse component-relative vertical position, in pixels
	 * @param index zero-relative index of currently selected item
	 * @param bDoubleClick true if mouse double-clicked on item
	 */
    protected void notifyHelper(int x, int y, int index, boolean bDoubleClick)
    {
        if (bDoubleClick)
        {
            sendActionEvent("DoubleClicked");
        }
        else
        {
            //inside image display area, and line has image!
            if (x >= 0 && (x - xCoord) < IMAGE_WIDTH)
            {
                ListItem listItem = (ListItem)items.elementAt(index);
                if (listItem.image != null)
                {
                    sendActionEvent("ImageSelected");
                }
            }
            sourceItemEvent();
        }
    }

	/**
	 * Used to calculate the border colors from the background color.
	 * @see #addNotify
	 * @see #setBackground
	 */
	protected void calculateBorderColors(Color c)
	{
		borderLighterColor	= Effect.hilight(c);
		borderDarkerColor	= Effect.shadow(c);
		borderLightColor	= Effect.darken(borderLighterColor, (double) 1 / 5);
		borderDarkColor		= Effect.darken(borderDarkerColor, (double) 1 / 5);
	}

	/**
	 * Determines the current index given a new absolute or relative index.
	 * @param offset the value used to compute the new index
	 * @param boolean true if offset is zero-relative, false if offset is
	 * relative to the current index value
	 * @return the new current index value
	 */
    protected int keyCalcIndex(int offset, boolean bAbsolute)
    {

        if (bAbsolute) lastIndex = offset;
        else           lastIndex += offset;

        if (lastIndex >= items.size()) lastIndex = items.size() - 1;
        else if (lastIndex < 0)        lastIndex = 0;

        return lastIndex;
    }


   /** Cherche l'item dont l'initiale est donne */
   public int moveCursor(int code, int start)
   {
      int m = items.size();

      code = Character.toLowerCase((char) code);

      for(int i = start; i < m; i++)
      {
        char c = Character.toLowerCase(getText(i).charAt(0));
        if(c == code) return i;
      }
     return -1;
   }

  /**
    Dfinit la couleur du fond de la liste diffrente de celle des bords
    DGS
   */

    public void setPanelColor(Color c) { PanelColor = c; }

    /**
       DGS
      */

    public void setStyle(Style s)
    {
      setBackground(s.background);
      setForeground(s.foreground);
      setFont(s.font);
    }

    public void setBackground(Color bg)
    {
      super.setBackground(bg);
      VBar.setBackground(bg);
      HBar.setBackground(bg);
    }

    public void setForeground(Color c)
    {
      super.setForeground(c);

      if(OS.isMacintosh()) return;

      textHighlightText =  Color.white; // fg highlight
      enabledColor      =  c;           // fg normal
      disabledColor     =  Color.gray;  // fg unactive
    }


    public synchronized void update(boolean d)
    {
      if(d) bAllDirty = true;
      paint(getGraphics());
    }

    public void setPainting(boolean cond)
    {
      bInternalBlockPaint = !cond;   // blocked if true
    }


class Focus extends FocusAdapter implements java.io.Serializable
{
        public void focusGained(FocusEvent event)
        {
	  hasFocus = true;
          showSelection();
        }

        public void focusLost(FocusEvent event)
        {
	   hasFocus = false;
           hideSelection();
        }
}



	/**
	 * Sends an action performed event to any action listeners, as needed.
	 */
	protected void sendActionEvent(String command)
	{
	  if (actionListener != null)
		actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, new String(command)));
	}


    /** DGS
        Retourne l'index pour un lment
    */
    public int getIndex(String str)
     {
       ListItem li;
       int z = items.size();

       for (int i = 0; i < z; i++)
        {
          li = (ListItem) items.elementAt(i);
          if(li.sText.equalsIgnoreCase(str)) return i;
        }
       return -1;
     }

    /**
        DGS
        Selecte un lment
     */

     public void showIt(int index)
     {
      if(validIndex(index))
      {
       index = keyCalcIndex(index, ABSOLUTE);
       bInternalBlockPaint = false;
       setSelectedIndex(index, true);

       requestFocus();
      }
     }

     public void showIt(String name)
     {
      showIt(getIndex(name));
     }


    /** Fonction de marquage */
    int countTagged;
    int lastTagged;

    public synchronized void setTagged(int index, boolean state)
     {
      ListItem listItem = (ListItem)items.elementAt(index);
      listItem.tagged = state;
      listItem.bDirty = true;
      if(state) countTagged++; else countTagged--;
      lastTagged = index;
        //Disp.it(index + ") tag " + listItem.sText + " "  + listItem.tagged);
     }


    public boolean isTagged(int index)
    {
      if (validIndex(index))
        return ((ListItem)items.elementAt(index)).tagged;
      return false;
    }

    public boolean isTagged()
    {
      return (getTaggedIndex() != -1);
    }

    public int getTaggedIndex()
    {
      int n = items.size();

      for(int i = 0; i < n; i++)
      {
        if(((ListItem) items.elementAt(i)).tagged) return i;
      }
      return -1;
    }

    public String getTaggedItem()
    {
      return getText(getTaggedIndex());
    }


    public String[] getTaggedItems()
    {
      VectorString vs = new VectorString();

      int n = items.size();

      for(int i = 0; i < n; i++)
      {
        ListItem tempItem = (ListItem)items.elementAt(i);
        if (tempItem.bSelected || tempItem.tagged)
          vs.addElement(((ListItem) items.elementAt(i)).sText);
      }

     return vs.toArray();
    }


    /** Uniquement select, pas tagg */
    public boolean isSelected()
    {
      int n = items.size();

      for(int i = 0; i < n; i++)
      {
        ListItem tempItem = (ListItem) items.elementAt(i);
        if(tempItem.bSelected) return true;
        if(tempItem.deleted)   return true;
      }
      return false;
    }


    /** Cache la slection (convertit en marqus)  */

    public void hideSelection()
    {
       ListItem li;
       int s = items.size();

       for (int i = 0; i < s; i++)
        {
          li = (ListItem) items.elementAt(i);
          if(li.bSelected)
          {
           li.bSelected = false;
           li.bDirty = true;
           li.tagged = true;
          }
        }

      bInternalBlockPaint = false;
      repaint();
    }

    /** Montre la slection */
    public void showSelection()
     {
       ListItem li;
       int s = items.size();

       for (int i = 0; i < s; i++)
        {
          li = (ListItem) items.elementAt(i);
          if(li.tagged)
          {
           li.bSelected = true;
           li.tagged = false;
           li.bDirty = true;
          }
        }

      bInternalBlockPaint = false;
      repaint();
    }

    /** Efface tout marquage  et affiche */

    public void clean()
    {
     bInternalBlockPaint = false;
     bAllDirty = true;
     deselectAll();
    }

//    public void setHLoc(int x) { xCoord = x; }

    /** Zero-relative index of the item displayed in the top row */
    protected int   nTopRow = 0;
    /**
     * Width of the list content portion of this component, in pixels.
     * This excludes the border and vertical scroll bar, if present.
     */
    protected int   lWidth = 0;
    /** Height of a single list cell, in pixels */
    protected int   cellHt = 0;
    /** Top margin before the contents of the first list item, in pixels */
    protected int   yAdj = 0;
    /** Width of the left and right borders combined, in pixels */
    protected int       borderWidth = 4;
    /** Width of the left or right border, in pixels */
    protected int       halfBorderWidth = 2;

    ActionListener actionListener = null;
    ItemListener itemListener = null;

    /** Extra space at end of a line */
    protected final int LINE_SLOP = 4;
    /** The label of this listbox (not displayed) */
    protected String ilbLabel;
    /** The vertical scroll bar. It always exists event if not shown or showable  */
    protected Scrollbar   VBar = null;
    /** The horizontal scroll bar. It always exists event if not shown or showable */
    protected Scrollbar   HBar = null;
    /** A vector of the items in the list */
    protected Vector      items;
    /** The font used for drawing text */
    protected Font        font;
    /** The font metrics of the font used for drawing text */
    transient protected FontMetrics fm = null;
    /** If true the list is dirty (needs repaint) */
    transient protected boolean bAllDirty = true;
    /** It true multiple list items can be selected at the same time */
    protected boolean bMultipleSelections = false;
    /** If true the repainting of the control is prevented */
    protected boolean bBlockPaint = false;
    /** True if the vertical scroll bar is currently visible */
    protected boolean bVBarVisible = false;
    /** True if the horizontal scroll bar is currently visible */
    protected boolean bHBarVisible = false;
    /** True if repainting is temporarily disabled for efficiency purposes */
    transient protected boolean bInternalBlockPaint = false;
    /**
     * The number of rows to show when this component is automatically laid out.
     * If rows is less than or equal to zero, it will size to show all of the
     * existing list items when automatically laid out.
     */
    protected int     rowsToShow = -1;
    /** Number of columns to show when this is automatically laid out */
    protected int     colsToShow = 10;
    /** The number of currently visible lines in this list */
    transient protected int     visibleRows = 0;  // DGS c'tait -1
    /** The number of currently visible columns in this list */
    transient protected int     visibleCols = -1;
    /** The zero-relative index of the first visible list item */
    // ne sert qu' getVisibleIndex car setVisibleIndex ne s'en sert pas
    //protected int     visibleIndex = -1;
    /** The standard height of a line of text in the current font */
    transient protected int     fontHeight;
    /**
     * The typical distance from the base line to the bottom of most characters
     * in the current font.
     */
    transient protected int     fontDescent;
    /** Not used */
    transient protected int     lastDownModifiers = -1;
    /**
     * The zero-relative index of the currently selected item in the list.
     * If multiple items are selected, this is the index of the item most
     * recently selected. If no items are selected, the value is -1.
     */
    transient protected int     lastSelected = -1;
    /**
     * The zero-relative index of the item the mouse was last moved over,
     * dragged over, or clicked on. This field is only valid while the
     * mouse is over this component. When the mouse is not over this component
     * the value is -1.
     */
    transient protected int     lastIndex = -1;
    /**
     * The zero-relative index of the item the mouse was pressed on.
     * It is used when bMultipleSelections is false, and valid
     * between the time the mouse is pressed and released.
     * All other times it equals -1.
     */
    transient protected int     lastTempIndex = -1;
    /** The number of list items currently selected */
    transient protected int     countSelected = 0;
    /** The current border type of this DListBox  */
    protected int     borderType = 0;
    /** The width of the widest list item, in pixels */
    protected int     longestLineValue = 0;
    /** The width of scroll bars, in pixels  */
    transient protected int barSize = 17;
    /** The amount the list items are scrolled horizontally, in pixels */
    protected int     xCoord = 0;
    /**
     * The time at which the mouse was released, in milliseconds.
     * This is used to determine when a double-click occurs.
     */
    transient protected long    prevSelectTime = -1;
    /**
     * The zero-relative index of the row at which the mouse was
     * previously released.
     * This is used to determine when a double-click occurs.
     */
    transient protected long    prevSelectRow = -1;


    /**
     * If true the vertical or horizontal scrollbar will be made
     * visible when necessary, false if it will never be made visible.
     * @see #setShowVerticalScroll/Horiz
     */
    protected boolean bAllowShowVBar = true;
    protected boolean bAllowShowHBar = true;

    /**
     * If true this DListBox is in "ComboBox mode".
     * "ComboBox mode" allows the selection point to follow
     * the mouse even when the mouse button is not down.
     * This only applies when the DListBox is not allowing multiple
     * selections.
     */
    protected boolean bComboMode = false;
    /**
     * If true list items have borders by default.
     * Borders are always shown as a solid single-pixel wide line around
     * the cell.
     */
    protected boolean bCellBorders = false;

    /** Colors used in drawing of the border  */
    protected Color borderLighterColor	= null;
	protected Color borderDarkerColor	= null;
	protected Color borderLightColor	= null;
	protected Color borderDarkColor	= null;
    /** Cached value of the background color.
        Used to determine if calculated colors need to be updated */
	protected Color cachedBackground	= null;

	//!!! LAB !!! Come back and remove this isMacintosh stuff once MRJ handles SystemColor!

    /** The background and text color for highlighted text */
    protected Color textHighlight     = OS.isMacintosh() ? new Color(0,0,128) : SystemColor.textHighlight;
    protected Color textHighlightText = OS.isMacintosh() ? Color.white : SystemColor.textHighlightText;

    /** The default color for enabled and disabled text items */
    protected Color enabledColor = OS.isMacintosh() ? Color.black : SystemColor.textText;
    protected Color disabledColor = OS.isMacintosh() ? Color.gray : SystemColor.textInactiveText;
    protected int fastDownCount = 0;
    /**
     * True if drawing was handled on mouse down and no drawing action needs
     * to be taken on mouse up.
     */
    protected boolean bMouseDrawHandled = false;

//	private Mouse		mouse		= null;
	private MouseMotion	mouseMotion	= null;
//	private Key		key		= null;
//	private Adjustment	adjustment	= null;


    // Rajout

    // Couleur du paneau ou est place la liste
    protected Color PanelColor = Color.gray;

    protected Focus focus = null;
	public transient boolean hasFocus = false;

}

