/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.Image;
import java.io.*;
import common.*;

/**
 * This is a single node in the TreeView panel.
 * It displays text and optionally one of two images depending on its state,
 * collapsed or expanded.
 * It also may have an object associated with it that doesn't get displayed.
 */
public class DTreeNode implements java.io.Serializable
{
	DTreeNode	sibling;
	DTreeNode	child;
	DTreeNode	parent;
	String		text;
	transient Image	collapsedImage;
	transient Image	expandedImage;
	int			numberOfChildren;
	Object		dataObject;
	DTreeView	treeView;

	int			depth = -1;
	boolean	isExpanded = false;    // A des child
    boolean isDirectory = false;  // DGS Branche, peut avoir des child
    boolean isExpandable = false; // Branche vide mais complte en temp rel

	/**
	 * Constructs a default DTreeNode.
	 */
    public DTreeNode() { 		//{{INIT_CONTROLS
		//}}
}

    /**
     * Constructs a DTreeNode with the given text label.
     * @param text the text to display for this node
     * @param treeView the instance of TreeView whose node this is.
     * Typically "this"
     */
	public DTreeNode(String text, DTreeView treeView)
	{
	    this(text, null, null, treeView);
	}

    /**
     * Constructs a DTreeNode with the given text label, and collapsed and
     * expanded images.
     * @param text the text to display for this node
     * @param collapsedImage the image to use when this node is collapsed, hiding
     * all of its child nodes
     * @param expandedImage the image to use when this node is expanded, showing
     * all of its child nodes
     * @param treeView the instance of TreeView whose node this is.
     * Typically "this"
     */
	public DTreeNode(String text, Image collapsedImage, Image expandedImage, DTreeView treeView)
	{
	    this.text				= text;
	    this.sibling			= null;
	    this.child				= null;
	    this.collapsedImage 	= collapsedImage;
	    this.expandedImage  	= expandedImage;
	    this.numberOfChildren	= 0;
	    this.dataObject			= null;
	    this.treeView			= treeView;
	}




	/**
	 * Sets a new text label for this node.
	 */
	public void setText(String s)
	{
	    text = new String(s);
	    if (treeView != null) treeView.triggerRedraw();
	}

	/**
	 * Sets an (not displayed) object to associate with this node.
	 */
	public void setDataObject(Object theObject)
	{
	    dataObject = theObject;
	}
    /**
     * Notes the current depth of this node.
     */
	void setDepth(int depth) { this.depth = depth;}

	/**
	 * Gets the (not displayed) object associated with this node.
	 */
	public Object getDataObject() { return dataObject; }
	/**
	 * Gets the current text label for this node.
	 */
	public String getText() { return text;}
	/**
	 * Gets the parent of this node.
	 */
	public DTreeNode getParent() { return parent; }

	/**
	 * Gets the child of this node.
	 */
    public DTreeNode getChild()  { return child; }

	/**
	 * Gets the next sibling of this node.
	 */
    public DTreeNode getSibling() { return sibling; }


    /**
     * Gets the depth of this node as previously noted.
     */
	public int getDepth() { return depth; }

    /**
     * Determines whether this node is expanded.
     * A node is expanded if its child nodes are visible.
     * @return true if the node is expanded, false if it is collapsed
     */
	public boolean isExpanded() { return isExpanded; }

    /**
     * Determines whether this node is expandable.
     * A node is expandable if it has one or more child nodes.
     */
	public boolean isExpandable()
	{
	    return (child != null) || isExpandable;
	}

    /**
      DGS
      Si le Node est  complter
    */  
    public boolean isIncomplete()
    {
     return (child == null) && isDirectory;   
    }    

	/**
	 * Sets a flag indicating that this node is expanded, if it is expandable.
	 */
	public void expand()
	{
	 if (isExpandable())
	  {
	     isExpanded = true;
		 if (treeView != null)	treeView.triggerRedraw();
      }
	}

	/**
	 * Sets a flag indicating that this node is not expanded.
	 */
	public void collapse()
	{
	  isExpanded = false;
	  if (treeView != null)	treeView.triggerRedraw();
	}

	/**
	 * Toggles the node state between collapsed and expanded, if the node
	 * is expandable.
	 */
	public void toggle()
	{
	    if (isExpanded) collapse();
	    else            expand();
	}

	/**
	 * Gets the proper image for this node in its current state, expanded or collapsed.
	 * @return the current image for this node in its current state
	 */
	public Image getImage()
	{
	    return ((isExpanded && (expandedImage != null))
	            ? expandedImage
	            : collapsedImage);
	}

	/**
	 * Sets the image to use for this node when it is expanded.
	 * @param image the image to use when this node is expanded
	 * @see #setCollapsedImage
	 * @see #getImage
	 */
	public void setExpandedImage(Image image)
	{
	    expandedImage = image;
	    if (isExpanded() && treeView != null)
			treeView.triggerRedraw();
	}

	/**
	 * Sets the image to use for this node when it is not expanded.
	 * @param image the image to use when this node is collapsed
	 * @see #setExpandedImage
	 * @see #getImage
	 */
	public void setCollapsedImage(Image image)
	{
	    collapsedImage = image;
	    if (!isExpanded() && treeView != null)
			treeView.triggerRedraw();
	}
	
	
	public String toString()
	{
	  return "Node " + text;   
	}    

	//{{DECLARE_CONTROLS
	//}}
}
