/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import common.*;

/**
 * Creates an tree view of text headings and, optionally, images.
 * The headings are arranged in a hierarchical fashion, and can be
 * expanded to show their sub-headings or collapsed, hiding their
 * sub-headings.
 * A TreeNode object is used for each heading.
 */
public class DTreeView extends Panel implements ItemSelectable
{
	// constants for insertion
	/**
	 * Constant indicating that the new node is to be a child
	 * of the existing node.
	 * @see #insert
	 */
	public static final int CHILD   = 0;
	/**
	 * Constant indicating that the new node is to be the next
	 * sibling of the existing node.
	 * @see #insert
	 */
	public static final int NEXT    = CHILD + 1;
	/**
	 * Constant indicating that the new node is to be the last
	 * sibling of the existing node.
	 * @see #insert
	 */
	public static final int LAST    = CHILD + 2;

    public int selectedIndex;   // Message for caller
    public String selectedText; // Message for caller

	/**
	 * Constructs an empty DTreeView.
	 */
	public DTreeView()
	{
		super.setLayout(null);
	    verticalScrollBar = new Scrollbar(Scrollbar.VERTICAL);
        verticalScrollBar.setVisible(false);
        add(verticalScrollBar);
        horizontalScrollBar = new Scrollbar(Scrollbar.HORIZONTAL);
        horizontalScrollBar.setVisible(false);
        add(horizontalScrollBar);
		needResetVector = true;
	}

	/**
	 * Constructs a DTreeView with the given node.
	 * @param head the root node of the constructed tree
	 */
	public DTreeView(DTreeNode head)
	{
	    this();
	    selectedNode = rootNode = head;
	    count = 1;
	}


    /** Change font et couleurs */

    public void setStyle(Style s)
     {
      setFont(s.font);
      setBackground(s.background);
      setForeground(s.foreground);

      verticalScrollBar.setBackground(s.background);
      horizontalScrollBar.setBackground(s.background);

      triggerRedraw();
      invalidate();
     }


    /**
     * Returns the selected items or null if no items are selected.
     * A standard method of the ItemSelectable interface.
     */
    public Object[] getSelectedObjects()
    {
    	if (selectedNode == null) return null;

    	DTreeNode[] selectedObjects = new DTreeNode[1];
    	selectedObjects[0] = selectedNode;

    	return selectedObjects;
    }


	/** Clears the tree structure and redraws */

	public void clear()
	{
		rootNode = selectedNode = null;
	    count = 0;
	    v = new Vector();
	    e = new Vector();
		triggerRedraw();

		invalidate();
	}

    /**
     * Returns the "root" node.
     */
	public DTreeNode getRootNode() { return rootNode; }

	/** Returns the total number of nodes in the tree */
	public int getCount() { return count; }

	/**
	 * Returns the total number of viewable nodes in the tree.
     * A node is viewable if all of its parents are expanded.
     */
	public int getViewCount() { return viewCount; }

	/**
	 * Determines if the given node is viewable.
     * A node is viewable if all of its parents are expanded.
	 */
	boolean viewable(DTreeNode node)
	{
	    for (int i = 0; i < viewCount; i++)
          if (node == v.elementAt(i)) return true;

	    return false;
	}

	/**
	 * Determines if the node with the given text is viewable.
     * A node is viewable if all of its parents are expanded.
	 */
	boolean viewable(String s)
	{
	    if (s==null)  return false;

	    for (int i=0; i<viewCount; i++)
	    {
	      DTreeNode tn = (DTreeNode)v.elementAt(i);
	      if(tn.text != null)
	       if(s.equals(tn.text))  return true;
	    }

	    return false;
	}

	/**
	 * Determines if the given node is in the DTreeView.
     * @param node the node to check
	 */
	public boolean exists(DTreeNode node)
	{
	    recount();
        for (int i=0; i<count; i++)
	        if (node == e.elementAt(i)) return true;

        return false;
	}

	/**
	 * Determines if the node with the given text is in the DTreeView.
	 */

	public int existIndex;
	public boolean exists(String s)
	{
	    recount();
	    existIndex = -1;

	    if (s == null) return false;

	    for (int i = 0; i < count; i++)
	    {
	        DTreeNode tn = (DTreeNode)e.elementAt(i);

	        if(tn.text != null)
	        {
	          existIndex = i;
	          if(s.equals(tn.text)) return true;
	        }
	    }

	    return false;
	}

	/**
	 * Adds a new node at root level. If there is no root node, the given
	 * node is made the root node. If there is a root node, the given node
	 * is made a sibling of the root node.
	 */
	public void append(DTreeNode newNode)
	{
	    if (rootNode == null)
	    {
	        rootNode = newNode;
	        selectedNode = rootNode;
	        count = 1;
	        redrawTriggered = true;
	    }
	    else
	    {
	    	recount();
	    	if (e.contains(newNode))
	    		System.err.println(new Exception("append: Node already exists in tree."));
	    	else
		        addSibling(newNode, rootNode, true);
	    }
	}

	/**
	 * Adds the specified child node to the specified parent node
	 */
	public void addChild(DTreeNode newNode, DTreeNode relativeNode)
	{
	    if (relativeNode.child == null)
	    {
	        relativeNode.child = newNode;
	        newNode.parent = relativeNode;
	        count++;
	        redrawTriggered = true;
	    }
	    else
	    {
	        addSibling(newNode, relativeNode.child, true);
	    }

	    relativeNode.numberOfChildren++;
	}

	/**
	 * Adds the specified node as a sibling to the specified node
	 * @param newNode the node to add as a sibling
	 * @param siblingNode a sibling node to the new node.
	 */
	public void addSibling(DTreeNode newNode, DTreeNode siblingNode)
	{
		addSibling(newNode,siblingNode,true);
	}

	/**
	 * Adds the specified node as a sibling to the specified node
	 * @param newNode the node to add as a sibling
	 * @param siblingNode a sibling node to the new node.
	 * @param asLastSibling if true, then add the new node as the last.
	 */
	public void addSibling(DTreeNode newNode, DTreeNode siblingNode, boolean asLastSibling)
	{
		if (asLastSibling)
		{
		 // Find last sibling
         DTreeNode tempNode = siblingNode;
		 while (tempNode.sibling != null)
			tempNode = tempNode.sibling;

		 tempNode.sibling = newNode; // ajoute comme successeur au node trouv
		}
		else
		{
		 // Insert the newNode below the siblingNode
		 newNode.sibling = siblingNode.sibling;
         siblingNode.sibling = newNode; // successeur au node donn en param
		}

		//Set the parent of the new node to the parent of the sibling
		newNode.parent = siblingNode.parent;

		count++;
		redrawTriggered = true;
	}


	/**
	 * Inserts a new node relative to an existing node in the tree.
	 * @param newNode the new node to insert into the tree
	 * @param relativeNode the existing node used for a position reference
	 * @param position where to insert the new node relative to relativeNode.
	   position = CHILD inserts the new node as a child of the node
	   position = NEXT inserts the new node as the next sibling
	   position = LAST inserts the new node as the last sibling
	*/
	public void insert(DTreeNode newNode, DTreeNode relativeNode, int position)
	{
	    if (newNode == null || relativeNode == null)    return;

	    if (exists(relativeNode)==false)   return;

	    switch (position)
	    {
	        case CHILD: addChild(newNode, relativeNode);  break;
	        case NEXT:  addSibling(newNode, relativeNode, false); break;
	        case LAST:  addSibling(newNode, relativeNode, true);  break;
	        default:    return;  // invalid position
	    }
	}


	/**
	 * Removes the node with the given text from the DTreeView.
	 * @param s the node text to find
     * @return the DTreeNode removed from this DTreeView or null if not found
     * @see #removeSelected
	 */
	public DTreeNode remove(String s)
	{
		recount();

		for (int i=0; i<count; i++)
		{
			DTreeNode tn = (DTreeNode)e.elementAt(i);

			if (tn.text != null)
			{
			    if (s.equals(tn.text))
			    {
			        remove(tn);
			        redrawTriggered = true;
			        return tn;
			    }
			}
		}

		return null;
	}

	/**
	 * Removes the currently selected node from the DTreeView.
	 * Does not redraw the component.  Allows you to call repeatedly without
	 * causing the component to flicker while nodes are manipulated.  After all
	 * manipulation is done, repaint the tree.
     * @see #remove(java.lang.String)
	 */
	public void removeSelected()
	{
      if (selectedNode != null)  remove(selectedNode);
	}

	/**
	 * Removes the given node from the DTreeView.
	 * @param node the node to remove
     * @return the DTreeNode removed from this DTreeView or null if not found
	 * Does not redraw the component.  Allows you to call repeatedly without
	 * causing the component to flicker while nodes are manipulated.  After all
	 * manipulation is done, repaint the tree.
     * @see #remove(java.lang.String)
     * @see #removeSelected
	 */
	public void remove(DTreeNode node)
	{
	    if (!exists(node))  return;

	    if (node == selectedNode)
	    {
	        int index = v.indexOf(selectedNode);

	        if (index == -1) index = e.indexOf(selectedNode);//not viewable
	        if (index > viewCount-1) index = viewCount-1;
	        if (index > 0)
	        {
	            changeSelection((DTreeNode)v.elementAt(index-1));
	        }
	        else if (viewCount>1)
	        {
	            changeSelection((DTreeNode)v.elementAt(1));
	        }
	    }

	    // remove node and its decendents
	    if (node.parent != null)
	    {
	        if (node.parent.child == node)
	        {
	            if (node.sibling != null)
	            {
	                node.parent.child = node.sibling;
	            }
	            else
	            {
	                node.parent.child = null;
	                node.parent.collapse();
	            }
	        }
	        else
	        {
	            DTreeNode tn=node.parent.child;

	            while (tn.sibling != node)
	            {
	                tn = tn.sibling;
	            }

	            if (node.sibling != null)
	            {
	                tn.sibling = node.sibling;
	            }
	            else
	            {
	                tn.sibling = null;
	            }
	        }
	    }
	    else
	    {
	        if (node == rootNode)
	        {
	            if (node.sibling == null)
	            {
	                rootNode=null;
	            }
	            else
	            {
	                rootNode=node.sibling;
	            }
	        }
	        else
	        {
	            DTreeNode tn = rootNode;

	            while (tn.sibling != node)
	            {
	                tn = tn.sibling;
	            }

	            if (node.sibling != null)
	            {
	                tn.sibling = node.sibling;
	            }
	            else
	            {
	                tn.sibling = null;
	            }
	        }
	    }

	    recount();
	    redrawTriggered = true;
	}

	/**
	 * Gets the currently selected node.
	 * @return the currently selected node, or null if none selected
	 */
	public DTreeNode getSelectedNode() { return selectedNode; }

	/**
	 * Gets the text of the currently selected node.
	 * @return the text of the currently selected node or null if no node
	 * is selected
	 */
	public String getSelectedText()
	{
	    if (selectedNode == null)  return null;

	    return selectedNode.getText();
	}

    /**
     * Handles redrawing of this component on the screen.
     */
	public void update (Graphics g)	{ paint(g);	}

	/**
	 * Paints this component using the given graphics context.
	 */

	public void paint(Graphics g)
	{
		Dimension d = getSize();

        // redraw needed, or size has changed
		if (redrawTriggered || (d.width != viewWidth) || (d.height != viewHeight)) redraw(g);

        // Efface le fond / Enlev
		g.translate( -sbHPosition, 0);
		// DGS pas la couleur? D'ailleurs cet effacement est redondant
//        g.clearRect(sbHPosition, 0, d.width - sbVWidth, d. height - sbHHeight);
//        g.clearRect(sbHPosition + 1, 1, d.width - sbVWidth - 2, d. height - sbHHeight - 2);

        // C'est redondant
/*
        if (sbVShow && sbHShow)
        {
//          g.setColor(Color.lightGray);
            g.setColor(getBackground());
            g.fillRect(sbHPosition+d.width-sbVWidth, d.height-sbHHeight, sbVWidth, sbHHeight);
        }
*/

        g.clipRect(sbHPosition, 0, d.width - sbVWidth, d.height - sbHHeight);
//		g.drawImage(im1, 0, 0, this);
		g.drawImage(im1, 1, 1, this);  // DGS ne pas rafficher le bord

		// Border
		g.setColor(Color.black);
        //g.drawRect(sbHPosition, 0, d.width - sbVWidth - 1, d.height - sbHHeight - 1);
        // NB sera clipp  droite, inutile de calculer un offset
        g.drawRect(sbHPosition, 0, d.width - 1, d.height - 1);

        // Ces lignes n'affichent que le trait vertical!
        /*
        else
        {
         int xr = sbHPosition - d.width - sbVWidth - 1;
         int yb = d.height - sbHHeight - 1;

         g.drawLine(sbHPosition, 0, xr, 0);   // top
         g.drawLine(sbHPosition, 1, sbHPosition, yb);  // left
         g.drawLine(sbHPosition, yb, xr, yb);  // bottom

         int x[] = { xr, sbHPosition, sbHPosition, xr };
         int y[] = { 0, 0, yb, yb };
         g.drawPolyline(x, y, 4);
        }
		*/
	}


    /**
     * Lays out the vertical scrollbar as needed, then draws the DTreeView
     * into an offscreen image. This is used for cleaner refresh.
     * @param g the graphics object use for drawing
     */
	public void redraw(Graphics g)
	{
        Dimension d = getSize();

		redrawTriggered = false;

		if(needResetVector)	resetVector();
		else    			needResetVector = true;

        newWidth = compWidth(g);

        int inRectCount = ((d.height - sbHHeight) / cellSize);
	    if (viewCount > inRectCount)
	    {
  	        // need the vertical scrollbar
	        sbVShow = true;
	        sbVWidth = verticalScrollBar.getPreferredSize().width;
	    }
	    else
	    {
  	        sbVShow = false;
	        sbVWidth = 0;
	        sbVPosition = 0;
	    }

	    if (newWidth > (d.width-sbVWidth))
	    {
	        // need the horizontal scrollbar
	        sbHShow = true;
			sbHHeight = horizontalScrollBar.getPreferredSize().height;
	    }
	    else
	    {
	        sbHShow = false;
            sbHHeight = 0;
            sbHPosition = 0;
	    }

        drawTree();

	    if (sbVShow)
	    {
	        verticalScrollBar.setBounds(d.width-sbVWidth,0,sbVWidth,d.height-sbHHeight);
//	        verticalScrollBar.setValues(sbVPosition, inRectCount, 0, viewCount - ( isSun1_1 ? 0 : inRectCount));
	        verticalScrollBar.setValues(sbVPosition, inRectCount, 0, viewCount);
			verticalScrollBar.setBlockIncrement(inRectCount-1);
	        verticalScrollBar.setVisible(true);
	    }
	    else
	    {
	        verticalScrollBar.setVisible(false);
	    }

        if (sbHShow)
	    {
	        horizontalScrollBar.setBounds(0,d.height-sbHHeight,d.width-sbVWidth,sbHHeight);
//            horizontalScrollBar.setValues(sbHPosition, d.width-sbVWidth, 0, sbHSize-(isSun1_1?0:(d.width-sbVWidth)));
            horizontalScrollBar.setValues(sbHPosition, d.width-sbVWidth, 0, sbHSize - (d.width - sbVWidth));
            horizontalScrollBar.setBlockIncrement(d.width-sbVWidth);
            horizontalScrollBar.setUnitIncrement(sbHLineIncrement);
            horizontalScrollBar.setVisible(true);
	    }
	    else
	    {
	        horizontalScrollBar.setVisible(false);
	    }
	}

    /**
     * Draws the DTreeView into an offscreen image. This is used for cleaner refresh.
     */
	public void drawTree()
	{
		Dimension d = getSize();

		if(needResetVector)	resetVector();

		if ((d.width != viewWidth) || (d.height != viewHeight) || (g1==null) || (sbHSize!=newWidth))
		{
	        // size has changed, must resize image
//			im1 = createImage(Math.max(sbHSize=newWidth, d.width), d.height);
			im1 = createImage(Math.max(sbHSize = newWidth, d.width) - 2 - sbVWidth, d.height - 2); // DGS
			// DGS il faut -2 en largeur pour enlever le flicker
			// mais alors on a une double ligne quand le dernier drive h: est expans
			// et que l'on expand un autre drive quelconque

			if (g1 != null)	g1.dispose();
	        g1 = im1.getGraphics();
	        viewWidth = d.width;
	        viewHeight= d.height;
		}


	    Font f = getFont();  //unix version might not provide a default font

		//Make certain there is a font
	    if (f == null)
	    {
	        f = new Font("TimesRoman", Font.PLAIN, 13);
	        g1.setFont(f);
	        setFont(f);
	    }

	    //Make certain the graphics object has a font (Mac doesn't seem to)
		if (g1.getFont() == null) g1.setFont(f);

	    fm = g1.getFontMetrics();
	    g1.setColor(getBackground());
	    g1.fillRect(0, 0, im1.getWidth(this), d.height);// clear image

	    //do drawing for each visible node
	    int lastOne = sbVPosition + viewHeight / cellSize + 1;

	    if (lastOne > viewCount) lastOne = viewCount;

	    DTreeNode outerNode = null;

	    if (!v.isEmpty())
	        outerNode = (DTreeNode)v.elementAt(sbVPosition);

	    for (int i = sbVPosition; i < lastOne; i++)
	    {
	        DTreeNode node = (DTreeNode)v.elementAt(i);

	        //Disp.it("node drawn " + node.getText());

	        int x = cellSize * (node.depth + 1);
	        int y = (i - sbVPosition) * cellSize;

	        // draw lines
	        g1.setColor(getForeground());

	        // draw vertical sibling line
	        if (node.sibling != null)
	        {
	            int k = v.indexOf(node.sibling) - i;

	            if (k > lastOne) k = lastOne;

	            drawDotLine(x - cellSize/2, y + cellSize/2,
	                        x - cellSize/2, y + cellSize/2 +  k*cellSize);

	        }
	        // if sibling is above page, draw up to top of page for this level
	        for (int m = 0; m < i; m++)
	        {
	            DTreeNode sib = (DTreeNode)v.elementAt(m);

	            if ((sib.sibling == node) && (m < sbVPosition))
	            {
	                drawDotLine (x - cellSize/2, 0,
	                             x - cellSize/2, y + cellSize/2);
	            }
	        }

	        // draw vertical child lines
	        if (node.isExpanded())
	        {

	            drawDotLine(x + cellSize/2, y + cellSize -2 ,
	                        x + cellSize/2, y + cellSize + cellSize/2);
	        }
	        // draw node horizontal line
	        g1.setColor(getForeground());
	        drawDotLine(x - cellSize/2, y + cellSize/2,
	                    x + cellSize/2, y + cellSize/2);

	        // draw toggle box
	        drawNodeToggle(node, x, y);

	        // draw node image
	        Image nodeImage = node.getImage();

	        if (nodeImage != null)
	        {
	            g1.drawImage(nodeImage, x+imageInset, y, this);
	        }

	        // draw node text
	        if (node.text != null)
	        {
	            drawNodeText(node, y, node==selectedNode);
	        }

	        if(outerNode.depth>node.depth)
	            outerNode = node;
	    }

	    // draw outer vertical lines
	    if (outerNode!=null)
	    {
    	    while((outerNode = outerNode.parent) != null)
	        {
	            if (outerNode.sibling != null)
	                drawDotLine (cellSize*(outerNode.depth + 1) - cellSize/2, 0,
	                             cellSize*(outerNode.depth + 1) - cellSize/2, d.height);;
    	    }
        }
		needResetVector = true;
	}

	/**
	 * Used to draw the toggle box of an expandable node.
	 * Override to change the look of the toggle box.
	 */
	protected void drawNodeToggle(DTreeNode node, int x, int y)
	{
	    if (node.isExpandable())
	    {
	        g1.setColor(getBackground());
	        g1.fillRect(cellSize*(node.depth) + cellSize/4, y + clickSize/2, clickSize, clickSize );
	        g1.setColor(getForeground());
	        g1.drawRect(cellSize*(node.depth) + cellSize/4, y + clickSize/2, clickSize, clickSize );
	        // cross hair
	        g1.drawLine(cellSize*(node.depth) + cellSize/4 +2,             y + cellSize/2,
	                    cellSize*(node.depth) + cellSize/4 + clickSize -2, y + cellSize/2);

	        if (!(node.isExpanded()))
	        {
	            g1.drawLine(cellSize*(node.depth) + cellSize/2, y + clickSize/2 +2,
	                        cellSize*(node.depth) + cellSize/2, y + clickSize/2 + clickSize -2);
	        }
	    }
	}

    /**
	 * Returns the recommended dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the recommended size of this component.
     *
     * @see #getMinimumSize
	 */
	public synchronized Dimension getPreferredSize()
	{
    	Dimension p = getSize();
    	Dimension m = getMinimumSize();
    	return new Dimension(Math.max(p.width, m.width), Math.max(p.height, m.height));
	}


    /**
	 * Returns the minimum dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the minimum size of this component.
     *
     * @see #getPreferredSize
	 */
	public synchronized Dimension getMinimumSize()
	{
	    return new Dimension(20, 40);
	}


	/**
	 * Takes no action.
	 * Since layout managers CANNOT BE USED with this container the standard
	 * setLayout has been OVERRIDDEN for this container and does nothing.
	 *
	 * @param lm the layout manager to use to layout this container's components
	 * (IGNORED)
	 * @see java.awt.Container#getLayout
	 **/
	public void setLayout(LayoutManager lm) {	}

        public boolean isFocusable()     { return true; }

	/**
	 * Tells this component that it has been added to a container.
	 * This is a standard Java AWT method which gets called by the AWT when
	 * this component is added to a container. Typically, it is used to
	 * create this component's peer.
	 *
	 * It has been overridden here to hook-up event listeners.
	 *
	 * @see #removeNotify
	 */
	public synchronized void addNotify()
	{
		super.addNotify();

		//Hook up listeners
		if (mouse == null)
		{
			mouse = new Mouse();
			addMouseListener(mouse);
		}
		if (key == null)
		{
			key = new Key();
			addKeyListener(key);
		}
		if (adjustment == null)
		{
			adjustment = new Adjustment();
			verticalScrollBar.addAdjustmentListener(adjustment);
			horizontalScrollBar.addAdjustmentListener(adjustment);
		}
		if (focus == null)
		{
			focus = new Focus();
			addFocusListener(focus);
		}

	}

	/**
	 * Tells this component that it is being removed from a container.
	 * This is a standard Java AWT method which gets called by the AWT when
	 * this component is removed from a container. Typically, it is used to
	 * destroy the peers of this component and all its subcomponents.
	 *
	 * It has been overridden here to unhook event listeners.
	 *
	 * @see #addNotify
	 */
	public synchronized void removeNotify()
	{
		//Unhook listeners
		if (mouse != null)
		{
			removeMouseListener(mouse);
			mouse = null;
		}
		if (key != null)
		{
			removeKeyListener(key);
			key = null;
		}
		if (adjustment != null)
		{
			verticalScrollBar.removeAdjustmentListener(adjustment);
			horizontalScrollBar.removeAdjustmentListener(adjustment);
			adjustment = null;
		}
        if (focus != null)
		{
			removeFocusListener(focus);
			focus = null;
		}
		super.removeNotify();
	}

	/**
	 * Triggers redrawing the entire image, even if the size of the component
	 * has not changed.
	 */
	public void triggerRedraw()
	{
		redrawTriggered = true;
		repaint();
	}


    /**
       DGS
       complte l'arbo en temps rel
       A redfinir dans une classe hrite
    */

    protected void complete(DTreeNode node, int x, int y)  { };



	// -----------------------------------------
	// --------- event related methods ---------
	// -----------------------------------------

    /**
     * Adds the specified action listener to receive action events
     * from this button.
     * @param l the action listener
     */
	public void addActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.add(actionListener, l);
	}

    /**
     * Removes the specified action listener so it no longer receives
     * action events from this button.
     * @param l the action listener
     */
	public void removeActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, l);
	}

    /**
     * Add a listener to recieve item events when the state of
     * an item changes.
     * <p>
     * This is a standard method of the ItemSelectable interface.
     * @param l the listener to recieve events
     * @see ItemEvent
     */
    public void addItemListener(ItemListener l)
    {
        itemListener = AWTEventMulticaster.add(itemListener, l);
    }

    /**
     * Removes an item listener.
     * <p>
     * This is a standard method of the ItemSelectable interface.
     * @param l the listener being removed
     * @see ItemEvent
     */
    public void removeItemListener(ItemListener l)
    {
        itemListener = AWTEventMulticaster.remove(itemListener, l);
    }


	protected void drawNodeText(DTreeNode node, int yPosition, boolean eraseBackground)
	{
	    if (node == null)   return;

	    Color fg, bg;
	    int depth=node.depth;
	    Image nodeImage = node.getImage();
	    int textOffset = ((depth + 1) * (cellSize)) + cellSize + textInset - (nodeImage == null ? 12 : 0);

	    if (node == selectedNode && hasFocus)
	    {
	    	//??RKM?? Temp until these return some good values
		 if (OS.isMacintosh())
         {
           fg = Color.white;
           bg = new Color(0,0,128);
         }
		 else
         {
           fg = SystemColor.textHighlightText;
		   bg = SystemColor.textHighlight;
         }

	    }
	    else
	    {
	       fg = getForeground();
	       bg = getBackground();
	    }

	    if (eraseBackground)
	    {
	        g1.setColor(bg);
	        g1.fillRect(textOffset-1, yPosition+1, fm.stringWidth(node.text)+4, cellSize-1);
	    }

        if (node == selectedNode)
        {
            g1.setColor(getForeground());
            g1.drawRect(textOffset-1, yPosition+1, fm.stringWidth(node.text)+3, cellSize-2);
            repaint(Math.max(0,textOffset-1-sbHPosition), yPosition+1, fm.stringWidth(node.text)+4, cellSize-1);
        }
	    g1.setColor(fg);
	    g1.drawString(node.text, textOffset, yPosition + cellSize - textBaseLine);
	}

	/**
	 * Sends an action performed event to any action listeners, as needed.
	 */
/*
	protected void sendActionEvent()
	{
	  sendActionEvent("Enter")
	}
*/
	protected void sendActionEvent(String command)
	{
	  selectedText = new String(selectedNode.getText());
  	  if (actionListener != null)
		actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, command ));
	}

	protected void sendSelectionEvent(int index)
	{
      selectedIndex = index;
      if (actionListener != null)
		actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "Select"));
	}


    protected int compWidth(Graphics gg)
    {
        int size = 0;
        int textOffset;
        DTreeNode node;

   	    Font f = getFont();  //unix version might not provide a default font
		//Make certain there is a font
	    if (f == null)
	    {
	        f = new Font("TimesRoman", Font.PLAIN, 13);
	        if(gg != null)	gg.setFont(f);
	        setFont(f);
	    }

	    if(gg == null) 	fm = null;
	    else	    	fm = gg.getFontMetrics();

	    if(fm == null) 	fm = getFontMetrics(f);

		if(fm == null || v == null)	size = 100;
		else
		{
	        for (int i=0; i < v.size(); i++)
	        {
	            node = (DTreeNode)v.elementAt(i);
	       	    textOffset = ((node.depth + 1) * (cellSize)) + cellSize + textInset - (node.getImage()==null ? 12:0);
	       	    if (size < (textOffset+fm.stringWidth(node.text)+6))
	    	        size = textOffset+fm.stringWidth(node.text)+6;
	        }
		}

        return size;
    }

	protected void drawDotLine(int x0, int y0, int x1, int y1)
	{
	   if (y0==y1)
	   {
	        for (int i = x0; i<x1; i+=2)
	        {
	           g1.drawLine(i,y0, i, y1);
	        }
	    }
	    else
	    {
	        for (int i = y0; i<y1; i+=2)
	        {
	            g1.drawLine(x0, i, x1, i);
	        }
	    }
	}

	/**
	 * Handles selecting the given node.
	 * @param node the node to select
	 */
	protected void changeSelection(DTreeNode node)
	{
	    if (node == selectedNode)  return;

	    DTreeNode oldNode = selectedNode;
	    selectedNode = node;
	    drawNodeText(oldNode, (v.indexOf(oldNode) - sbVPosition)*cellSize, true);
	    drawNodeText(node,    (v.indexOf(node)    - sbVPosition)*cellSize, true);
	    // send select event

	    int index = v.indexOf(selectedNode);

		if (itemListener != null)
		{
			itemListener.itemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, oldNode, ItemEvent.DESELECTED));
			itemListener.itemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, selectedNode, ItemEvent.SELECTED));
		}

	    if (index < sbVPosition)
	    { //scroll up
	        sbVPosition--;
	        verticalScrollBar.setValue(sbVPosition);
	        triggerRedraw();
	        return;
	    }

	    if (index >= sbVPosition + (viewHeight-cellSize/2)/cellSize)
	    {
	        sbVPosition++;
	        verticalScrollBar.setValue(sbVPosition);
	        triggerRedraw();
	        return;
	    }

       sendSelectionEvent(index);
	   repaint();
	}


	/**
       DGS
       Comme changeSelection avec des index absolus
	 */
	protected void absoluteSelection(DTreeNode node)
	{
	    if (node == selectedNode)  return;

	    DTreeNode oldNode = selectedNode;
	    selectedNode = node;
	    drawNodeText(oldNode, (v.indexOf(oldNode) - sbVPosition)*cellSize, true);
	    drawNodeText(node,    (v.indexOf(node)    - sbVPosition)*cellSize, true);
	    // send select event

	    int index = v.indexOf(selectedNode);

		if (itemListener != null)
		{
			itemListener.itemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, oldNode, ItemEvent.DESELECTED));
			itemListener.itemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, selectedNode, ItemEvent.SELECTED));
		}

	    if (index < sbVPosition)
	    { //scroll up
	        sbVPosition = index;
	        verticalScrollBar.setValue(sbVPosition);
	        triggerRedraw();
	        return;
	    }

	    if (index >= sbVPosition + (viewHeight-cellSize/2)/cellSize)
	    {
	        sbVPosition = index;
	        verticalScrollBar.setValue(sbVPosition);
	        triggerRedraw();
	        return;
	    }

        sendSelectionEvent(index);
	    repaint();
	}

	/**
	 * Generates a new tree node.
	 * Override if you want to return your own instantiation of a DTreeNode subclas.
     * Chang pour ajouter des images
	 * @param text the node text
	 * @param treeView a reference to the parent DTreeView (usually "this").
	 * @return a new DTreeNode
	 */
	protected DTreeNode newTreeNode(String text, DTreeView treeView)
	{
	  return new DTreeNode(text, treeView);
	}

    /**
       Gestion du Vector contenant les nodes
       Reconstruit un Vector des nodes
    */
	protected void recount()
	{
	    count = 0;
	    e = new Vector();

	    if (rootNode != null)
	    {
	        rootNode.depth=0;
	        traverse(rootNode);
	    }
	}

    /**
     LOCAL
	 Traverses tree to put nodes into vector v
	 for internal processing. Depths of nodes are set,
	 and viewCount and viewWidest is set.
	*/

	protected void traverse(DTreeNode node)
	{
	    count++;
	    e.addElement(node);

	    if (node.child != null)
	    {
	        node.child.depth = node.depth+1;
	        traverse(node.child);
	    }
	    if (node.sibling != null)
	    {
	        node.sibling.depth = node.depth;
	        traverse(node.sibling);
	    }
	}


    /*
       Clear vector
    */
	protected void resetVector()
	{
		v = new Vector(count);
		viewWidest = 30;

		if (count < 1) { viewCount = 0;	return;	}

		rootNode.depth = 0;
		vectorize(rootNode, true, v);
		viewCount = v.size();

		needResetVector = false;
	}

    /*
       Build list of nodes
    */
	protected void vectorize (DTreeNode node, boolean respectExpanded, Vector nodeVector)
	{
	  if (node == null)  return;

      nodeVector.addElement(node);

	  if ((!respectExpanded && node.child != null) || node.isExpanded())
	   {
        node.child.depth = node.depth + 1;
        vectorize(node.child,respectExpanded,nodeVector);
	   }

	  if (node.sibling != null)
	   {
        node.sibling.depth = node.depth;
        vectorize(node.sibling,respectExpanded,nodeVector);
	   }

	}

	protected void debugVector()
	{
	    //int vSize = v.size();

	    for (int i=0; i<count; i++)
	    {
	        DTreeNode node = (DTreeNode) v.elementAt(i);
	        System.out.println(node.text);
	    }
	}

    /**
       DGS
       retourne l'index du node selected
    */

    public int getSelectedIndex()
    {
     if(selectedNode != null)
      {
       for (int index = 0; index < viewCount; index++)
        {
         if (selectedNode == v.elementAt(index))
            return index;
	    }
	  }
	 return 0;
    }



	int		sbVPosition				= 0;	// hold value of vertical scrollbar
	int		sbVWidth;    			   		// width of vertical scrollbar
	int		sbHPosition				= 0;	// hold value of horizontal scrollbar
	int		sbHHeight				= 0;	// height of horizontal scrollbar
	long	sbVTimer				= -1;	// time of last vert scrollbar event
	int		cellSize   				= 16;   // size of node image
	int		clickSize   			= 8;    // size of mouse toggle (plus or minus)
	int		imageInset				= 3;    // left margin of node image
	int		textInset   			= 6 + 8;    // left margin for text
	int		textBaseLine			= 3;    // position of font baseline from bottom of cell
	int		doubleClickResolution	= 333;	// double-click speed in milliseconds

	/** root node of tree */
	protected DTreeNode rootNode;
	/** highlighted node */
	protected DTreeNode selectedNode;
	/** first node in window */
	protected DTreeNode topVisibleNode;
	/** The vertical scrollbar */
	protected Scrollbar	verticalScrollBar;
	/** show or hide vertical scrollbar */
	protected boolean sbVShow = false;
	/** Number of nodes in the tree */
	protected int count = 0;
	/**
	 * Number of viewable nodes in the tree.
	 * A node is viewable if all of its parents are expanded.
	 */
	protected int	viewCount = 0;
	/** The horizontal scrollbar */
    protected Scrollbar horizontalScrollBar;
	/** size of horizontal scrollbar */
    protected int sbHSize;
	/** for horizontal scrollbar */
    protected int newWidth = 0;
	/** show or hide horizontal scrollbar */
   	protected boolean sbHShow = false;
   	/**
   	 * Keeps track of if drawTree needs to call resetVector or not.
   	 */
   	protected boolean needResetVector;

	protected int sbHLineIncrement = 4;
   	/** pixel size of tree display */
	protected int viewHeight = 300;
   	/** pixel size of tree display */
	protected int viewWidth  = 300;
   	/** widest item displayable (for horz scroll) */
	protected int viewWidest = 0 ;
    /** The this component's key event listener */
	protected Key key = null;
    /** The component's mouse event listener */
	protected Mouse mouse = null;
    /** The component's adjustment event listener */
	protected Adjustment adjustment = null;
    /** The action listener to keep track of listeners for our action event */
	protected ActionListener actionListener = null;
    /** The item listener to keep track of listeners for our item event */
    protected ItemListener itemListener = null;
    /** The component's focus event listener */
    protected Focus focus = null;
	/** is vector of viewable nodes */
	protected Vector v;
	/** e is vector of existing nodes */
	protected Vector e;
    /**
     * Flag indicating repaint() should redraw the entire image even
     * if the component size has not changed.
     */
	transient protected boolean redrawTriggered = false;
	transient boolean hasFocus = false;
   	/** current font metrics */
	transient protected FontMetrics fm;
   	/** save time of last mouse down (for double click) */
	transient long timeMouseDown;
	/** Checks for scrollbars that have different max value */
    transient protected boolean isSun1_1;
	/**
	 * Offscreen Image used for buffering the painting process.
	 */
	transient protected Image im1;
	/**
	 * Offscreen graphics context used for buffering the painting process.
	 */
	transient protected Graphics g1 = null;

	class Adjustment implements AdjustmentListener, java.io.Serializable
	{
		public void adjustmentValueChanged(AdjustmentEvent event)
		{
			handleAdjustmentEvent(event);
		}
	}

	class Mouse extends MouseAdapter
	{
	    /**
	     * Processes MOUSE_DOWN events.
	     * This is a standard Java AWT method which gets called by the AWT
	     * method handleEvent() in response to receiving a MOUSE_DOWN
	     * event. These events occur when the mouse button is pressed while
	     * inside this component.
	     *
	     * @param event the event
	     * @param x the component-relative horizontal coordinate of the mouse
	     * @param y the component-relative vertical coordinate of the mouse
	     * @see java.awt.Component#mouseUp
	     */
		public void mousePressed(MouseEvent event)
		{
			handleMousePressed(event);
		}

		public void mouseReleased(MouseEvent event) { }
	}

	class Key extends KeyAdapter implements java.io.Serializable
	{
	    /**
	     * Processes KEY_PRESS and KEY_ACTION events.
	     * This is a standard Java AWT method which gets called by the AWT
	     * method handleEvent() in response to receiving a KEY_PRESS or
	     * KEY_ACTION event. These events occur when this component
         * has the focus and the user presses a "normal" or an
         * "action" (F1, page up, etc) key.
	     *
	     * @param event the Event
	     * @param key the key that was pressed
	     * @return true if the event was handled
	     * @see java.awt.Component#keyUp
	     * @see #handleEvent
	     */
		public void keyPressed(KeyEvent event)
		{
			handleKeyPressed(event);

		}
	}

	class Focus extends FocusAdapter implements java.io.Serializable
	{
       public void focusGained(FocusEvent event)
        {
	     hasFocus = true;
	     if (selectedNode != null && v != null)
	        drawNodeText(selectedNode,    (v.indexOf(selectedNode) - sbVPosition)*cellSize, true);
     	}

       public void focusLost(FocusEvent event)
       {
	    hasFocus = false;
	    if (selectedNode != null && v != null)
	        drawNodeText(selectedNode,    (v.indexOf(selectedNode) - sbVPosition)*cellSize, true);
	   }

	}

	/**
	 * Handles mouse pressed events.
	 * This function will get called when the component recives a mouse pressed event.
	 * Override to change the way mouse pressed is handled.
	 * @param event the MouseEvent
	 */
	protected void handleMousePressed(MouseEvent event)
	{
		requestFocus();

		int x = event.getX();
		int y = event.getY();

		int index = (y / cellSize) + sbVPosition;

		//If clicked below the last node
		if (index > viewCount-1) return;

		DTreeNode oldNode = selectedNode;
		DTreeNode newNode = (DTreeNode)v.elementAt(index);

		int newDepth = newNode.getDepth();

		// check for toggle box click
		Rectangle toggleBox = new Rectangle(cellSize*newDepth + cellSize / 4,
						                    (index-sbVPosition) * cellSize + clickSize / 2,
						                    clickSize, clickSize);
        // Si on clique sur le carr avec + -
		if (toggleBox.contains(x,y))
		{
		    complete(newNode, x, y); // DGS
			newNode.toggle();
			resetVector();
			if(!newNode.isExpanded())
			{
			 if (!v.contains(selectedNode)) changeSelection(newNode);
			}
			triggerRedraw();
			invalidate();
			//sendActionEvent();
		}
		else
		{
			changeSelection(newNode);

			// check for double click
			//long currentTime = event.getWhen();

			if ((newNode==oldNode) && ((event.getWhen() - timeMouseDown)<doubleClickResolution))
			{
				newNode.toggle();
				resetVector();
				triggerRedraw();
				invalidate();
				//sendActionEvent();
				return;
			}
			else
			{
				//single click action could be added here
				timeMouseDown = event.getWhen();
			}

		}
	}


	/**
	 * Handles adjustment events.
	 * This function will get called when the component recives a adjustment event.
	 * Override to change the way adjustment is handled.
	 * @param event the AdjustmentEvent
	 */
	protected void handleAdjustmentEvent(AdjustmentEvent event)
	{
		if (event.getAdjustable() == verticalScrollBar)
		{
			if (sbVPosition != verticalScrollBar.getValue())
			{
				sbVPosition = verticalScrollBar.getValue();
				triggerRedraw();
			}
		}
		else
		if (event.getAdjustable() == horizontalScrollBar)
		{
			if (sbHPosition != horizontalScrollBar.getValue())
			{
				sbHPosition = horizontalScrollBar.getValue();
				triggerRedraw();
			}
		}
	}

	/**
	 * Handles key pressed events.
	 * This function will get called when the component recives a key pressed event.
	 * Override to change the way key pressed is handled.
	 * @param event the KeyEvent
	 */
	protected void handleKeyPressed(KeyEvent event)
	{
		int index = v.indexOf(selectedNode);
        int h = viewHeight / cellSize;
        //Disp.it("tree key code " + event.getKeyCode());

		switch (event.getKeyCode())
		{
		    case KeyEvent.VK_ENTER:    //enter key
		          sendActionEvent("Enter");
		          requestFocus();
		          break;
            case KeyEvent.VK_DELETE:
                  sendActionEvent("Del");
                  requestFocus();
                  break;

            case KeyEvent.VK_SPACE:
                 if(event.isControlDown() || event.isAltDown() || event.isShiftDown())
                   break;//super.handleKeyPressed(event);

		        if (selectedNode.isExpandable() && (!selectedNode.isExpanded()))
		        {
		            selectedNode.toggle();
		            complete(selectedNode, 0, 0);
		            triggerRedraw();
		            break;
		        }
		        if (selectedNode.isExpanded())
		        {
		            selectedNode.toggle();
		            triggerRedraw();
		            break;
		        }
     	        break;

		    case KeyEvent.VK_LEFT:    //left arrow
		        if(event.isControlDown())
                {
	                if(sbHPosition > 0)
                    {
                        horizontalScrollBar.setValue(Math.max(sbHPosition -= sbHLineIncrement,0));
                        repaint();
                    }
                    break;
	            }
                else
                {
                 index = v.indexOf(selectedNode.getParent());
                 absoluteSelection((DTreeNode) v.elementAt(index));
                 requestFocus();
                 break;
                }

		        // else drop through to "UP" with no "break;"
		    case KeyEvent.VK_UP:
		        if (index > 0)
		        {
		            index--;
		            changeSelection((DTreeNode)v.elementAt(index));
		            requestFocus();
		        }
		        break;
		    case KeyEvent.VK_RIGHT:
		        if(event.isControlDown())
    	        {
//	                int max = horizontalScrollBar.getMaximum()-(isSun1_1?size().width-sbVWidth:0);
	                int max = horizontalScrollBar.getMaximum() - (getSize().width - sbVWidth);
	                if(sbHShow && sbHPosition < max)
                    {
                        horizontalScrollBar.setValue(Math.min(sbHPosition+=sbHLineIncrement, max));
                        repaint();
                    }
                    break;
	            }
	            else
		        if (selectedNode.isExpandable() && (!selectedNode.isExpanded()))
		        {
		            selectedNode.toggle();
		            //sendActionEvent();
		            complete(selectedNode, 0, 0);
		            triggerRedraw();
		            break;
		        }

		        if (!selectedNode.isExpandable())
		        {
		            break;
		        }
		        // else drop thru' to DOWN
		    case KeyEvent.VK_DOWN:
		        if (index < viewCount-1)
		        {
		            index++;
		            changeSelection((DTreeNode)v.elementAt(index));
		            requestFocus();
		        }
		        break;

            case KeyEvent.VK_HOME:
		        if (index > 0)
		        {
		            index = 0;
		            absoluteSelection((DTreeNode) v.elementAt(index));
		            requestFocus();
		        }
		        break;

            case KeyEvent.VK_PAGE_UP:
               if(index > sbVPosition)
		        {
		          index = sbVPosition;
		          absoluteSelection((DTreeNode)v.elementAt(index));
		          requestFocus();
		        }
		       else
		        {
		          index = Math.max(0, index - h);
                  absoluteSelection((DTreeNode)v.elementAt(index));
		          requestFocus();
		        }
               break;

            case KeyEvent.VK_PAGE_DOWN:
  	            int lastOne = sbVPosition + h - 1;
                if(index < lastOne) index = lastOne;
		        else    	        index = index + h;
 	            absoluteSelection((DTreeNode)v.elementAt(index));
		        requestFocus();
                break;

            case KeyEvent.VK_END:

               if(index < viewCount-1)
		        {
		            index = viewCount-1;
		            absoluteSelection((DTreeNode)v.elementAt(index));
		            requestFocus();
                    break;
		        }

                else if(index < count-1)
                {
                   index = count - 1;
                   absoluteSelection((DTreeNode)v.elementAt(index));
                   requestFocus();
                }
		        break;
            /*
            case KeyEvent.VK_ESCAPE:
                deselect();
                break;
            */
            default: break;

		}
	}


}

