/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Graphics;
import common.*;

/**
 * The DirButton is a button component that has an arrow drawn in it that
 * points one of four ways (left, up, right, or down). At runtime, the button has
 * a raised look and a pressed look.
 * This component is usually used in conjunction with a combo or list box to
 * indicate a list that the user can view by clicking the arrow, or with spinners
 * to let the user scroll through available values.
 */
public class DirButton extends DButtonBase implements java.io.Serializable
{
    
    public static final int LEFT = 0;/** The point LEFT style constant */
    public static final int RIGHT = 1;/** The point RIGHT style constant */
    public static final int UP = 2;/** The point UP style constant */
    public static final int DOWN = 3;/** The point DOWN style constant */

    /** Constructs a default DirButton */
    public DirButton() { this(DOWN); }

    /**
     * Constructs a DirButton pointing the specified direction.
     * @param d a style constant indicating which direction to point the button
     */
	public DirButton(int d)
	{
		direction = d;
		left      = 0;
		right     = 0;
		bottom    = 0;
		setArrowColor(Color.black);
	}

    /** Sets the direction of the arrow after construction */

	public void setDirection(int d) { direction = d;}

    /** Returns the direction the button is currently pointing */

    public int getDirection() { return direction; }


	/** Sets the color of the direction arrow */
	public void setArrowColor(Color newValue)
	{
		arrowColor = newValue;
		disabledArrowColor = Color.lightGray;
	}

    /** Gets the current color of the direction arrow  */
	public Color getArrowColor() { return arrowColor; }



    /**
     * Sets the extra amount, in pixels, to shrink the arrow triangle.
     * Constrains the values such that the arrow will never be less than
     * three pixels.  If a value is entered that would exceed this limit,
     * the limit will be used instead.
     */


    /**
	 * Returns the recommended dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the recommended size of this component.
     *
     * @return a button that has a content area of 7 by 7 pixels.
     * @see java.awt.Component#getMinimumSize
	 */
    public Dimension getPreferredSize()
    {
    	Dimension defaultSize = super.getPreferredSize();

		return new Dimension(defaultSize.width + 5, defaultSize.height + 5);
    }

    /**
	 * Returns the minimum dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the minimum size of this component.
     *
     * @return a button that has a content area of 3 by 3 pixels.
     * @see java.awt.Component#getMinimumSize
	 */
    public Dimension getMinimumSize()
    {
    	Dimension defaultSize = super.getPreferredSize();

		return new Dimension(defaultSize.width + 3, defaultSize.height + 3);
    }

	/**
	 * Maintains the buttonImage size and draws the
	 * button in the buttonImage offscreen image.
	 * @see ButtonBase#updateButtonImage
	 */
	protected void updateButtonImage()
	{
		super.updateButtonImage();  // cre buttonImage
		Graphics g		= buttonImage.getGraphics();
	    Dimension s		= getSize();
        int trueBevel	= bevel + 1;
        if(blackBorder) trueBevel++;
        int centerHorizontal;
        int centerVertical;

        int topSide;
        int bottomSide;
        int leftSide;
        int rightSide;

        if(isEnabled()) {  g.setColor(arrowColor); }
        else       {       g.setColor(disabledArrowColor);  }

		centerHorizontal	= ((s.width - 1) / 2)					+ pressedAdjustment;
		centerVertical		= ((s.height - 1) / 2)					+ pressedAdjustment;
        
        int tiers;
        int huit;
        switch(direction)
        {
         case UP:
         case DOWN:
                    tiers = s.width / 3;
                    huit = s.width / 8;
                    top  = tiers;
                    bottom = tiers;
                    left = huit;
                    right = huit;
                    break;
         case LEFT:
         case RIGHT:
                    tiers = s.height / 3;
                    huit = s.height / 8;
                    top  = huit;
                    bottom = huit;
                    left = tiers;
                    right = tiers;
                    break;
        } 
            
		topSide				= (top + trueBevel - 1)					+ pressedAdjustment;
		bottomSide			= (s.height - bottom - trueBevel)	        + pressedAdjustment;
		leftSide			= (left + trueBevel - 1)				+ pressedAdjustment;
		rightSide			= (s.width - 1 - right - trueBevel)		+ pressedAdjustment;

		if (OS.isMacintosh()) { leftSide++; topSide++; }
         
        switch (direction)
		{
			case UP:
			{
                int hParity = rightSide - leftSide;
                if(hParity == ((hParity / 2) * 2)) rightSide--;

				fillTriangle(g, centerHorizontal, topSide, leftSide, bottomSide, rightSide, bottomSide);
				break;
			}
			case DOWN:
			{
			    // Il faut une largeur impaire pour une flche isocle 
                int hParity = rightSide - leftSide;
                if(hParity == ((hParity / 2) * 2)) rightSide--;
				fillTriangle(g, centerHorizontal, bottomSide, leftSide, topSide, rightSide, topSide);
				//       left, top                   right, top
				//           
				//                 center, bottom
				break;
			}

			case LEFT:
			{
                int vParity = bottomSide - topSide;
                if(vParity == ((vParity / 2) * 2)) bottomSide--;

				fillTriangle(g, leftSide, centerVertical, rightSide, bottomSide, rightSide, topSide);
				break;
			}

			case RIGHT:
			{
                int vParity = bottomSide - topSide;
                if(vParity == ((vParity / 2) * 2)) bottomSide--;

				fillTriangle(g, rightSide, centerVertical, leftSide, bottomSide, leftSide, topSide);
				break;
            }
		}
		if (g != null)  g.dispose();
	}

	/**
	   DGS Utilise polyLine
	   
	 * Fills a triangle which has at least one side that is straight up and down or left and right.
	 * @param g the Graphics to use to draw with.
	 * @param tipX the horizontal coordinate of the point opposite a straight side.
	 * @param tipY the vertical coordinate of the point opposite a straight side.
	 * @param aX the horizontal coordinate of one of the two points defining the straight side.
	 * @param aY the vertical coordinate of one of the two points defining the straight side.
	 * @param bX the horizontal coordinate of one of the two points defining the straight side.
	 * @param bY the vertical coordinate of one of the two points defining the straight side.
	 * @param direction the direction of the straight line UP, DOWN, or LEFT, RIGHT.
	 *
	 * aX and bX should be the same for UP or Down.  aY and bY should be the same for LEFT or RIGHT.
	 * If not, then the a coordinates are used.
	 */
	protected void fillTriangle(Graphics g, int tipX, int tipY, int aX, int aY, int bX, int bY)
	{
        int xp[] = { tipX, aX, bX, tipX };
		int yp[] = { tipY, aY, bY, tipY };
		g.drawPolyline(xp, yp, 4);
	}

    /**
     * Is the given bevel size valid for this button.
     * @param i the given bevel size
     * @return true if the given bevel size is acceptable, false if not.
     */
     
    protected boolean isValidBevelSize(int i)
    {
        Dimension s = getSize();

        int temp = i * 2 + 4;

        return !(i < 0 || s.width < temp || s.height < temp);
    }

    /**
     * Is the given direction valid for this button.
     * @param i the given bevel size
     * @return true if the given direction is acceptable, false if not.
     */
    protected boolean isValidDirection(int i)
    {
    	switch(i)
    	{
    		case LEFT:
    		case RIGHT:
    		case UP:
    		case DOWN:	return true;
    		default:	return false;
    	}
    }


	/** The color of the arrow in the button */
	protected Color	arrowColor			= null;
	/** The color of the arrow when the button is disabled */
	protected Color	disabledArrowColor	= null;

	/** The direction the arrow points One of: LEFT, UP, RIGHT, or DOWN */
	protected int		direction;
	/**
	 * The number of pixels to shrink the arrow from the left side of the button.
	 */
	protected int		left;
	protected int		right;
	protected int		top;
	protected int		bottom;

}
