/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.Color;

/** Many useful color related utility functions for color manipulation */
public class Effect
{
    /** Do not use, this is an all-static class  */
    public Effect() { 		//{{INIT_CONTROLS
		//}}
}

	/**
	 * Darkens a given color by the specified percentage.
	 * @param r The red component of the color to darken.
	 * @param g The green component of the color to darken.
	 * @param b The blue component of the color to darken.
	 * @param percent percentage to darken.  Needs to be <= 1 && >= 0.
	 * @return a new Color with the desired characteristics.
	 */
	public static Color darken( int r, int g, int b, double percent )
	{
		return new Color( Math.max((int)(r * (1-percent)), 0),
							Math.max((int)(g * (1-percent)),0),
							Math.max((int)(b * (1-percent)),0));
	}

	/**
	 * Darkens a given color by the specified percentage.
	 * @param to the Color to darken.
	 * @param percent percentage to darken.  Needs to be <= 1 && >= 0.
	 * @return a new Color with the desired characteristics.
	 */
	public static Color darken( Color c, double percent )
	{
	  return darken( c.getRed(), c.getGreen(), c.getBlue(), percent );
	}

	/**
	 * Lightens a given color by the specified percentage.
	 * @param r The red component of the color to lighten.
	 * @param g The green component of the color to lighten.
	 * @param b The blue component of the color to lighten.
	 * @param percent percentage to lighten.  Needs to be <= 1 && >= 0.
	 * @return a new Color with the desired characteristics.
     * @exception IllegalArgumentException
     * if the specified percentage value is unacceptable
	 */
	public static Color lighten( int r, int g, int b, double percent )
	{
		int r2, g2, b2;
		r2 = r + (int)((255 - r) * percent );
		g2 = g + (int)((255 - g) * percent );
		b2 = b + (int)((255 - b) * percent );
		return new Color( r2, g2, b2 );
	}

	/**
	 * Lightens a given color by the specified percentage.
	 * @param to the Color to lighten.
	 * @param percent percentage to lighten.  Needs to be <= 1 && >= 0.
	 * @return a new Color with the desired characteristics.
	 */
	public static Color lighten( Color c, double percent )
	{
	  return lighten(c.getRed(), c.getGreen(), c.getBlue(), percent );
	}

	/**
	 * Fades from one color to another by the given percentage.
	 * @param from the Color to fade from.
	 * @param to the Color to fade to.
	 * @param percent percentage to fade.  Needs to be <= 1 && >= 0.
	 * @return a new Color with the desired characteristics.
	 */
	public static Color fade( Color from, Color to, double percent )
	{
		int from_r, from_g, from_b;
		int to_r, to_g, to_b;
		int r, g, b;

		from_r = from.getRed();
		from_g = from.getGreen();
		from_b = from.getBlue();

		to_r = to.getRed();
		to_g = to.getGreen();
		to_b = to.getBlue();

		if (from_r > to_r)
			r = to_r + (int)((from_r - to_r)* (1 - percent));
		else
			r = to_r - (int)((to_r - from_r)* (1 - percent));
		if (from_g > to_r)
			g = to_g + (int)((from_g - to_g)* (1 - percent));
		else
			g = to_g - (int)((to_g - from_g)* (1 - percent));
		if (from_b > to_b)
			b = to_b + (int)((from_b - to_b)* (1 - percent));
		else
			b = to_b - (int)((to_b - from_b)* (1 - percent));

		return new Color(r, g, b);
	}

	/**
	 * Given a Color this function determines the lightness percent.
	 * @param c The Color to calculate from.
	 * @return the percent light of the specified color. 
	 * This value will be >= 0 && <= 1.
	 */
	public static double lightness(Color c)
	{
		if(c == null)	return 0;

		double r, g, b, max, min;
		r = c.getRed();
		g = c.getGreen();
		b = c.getBlue();
		max = (Math.max(r, Math.max(g, b)) / 255) / 2;
		min = (Math.min(r, Math.min(g, b)) / 255) / 2;
		return (max + min);
	}

	/**
	 * Used to calculate a hilight color from a given color.
	 * @param c The color to use in the calculation. 
	 * @return the newly calculated hilight color.
	 */
	public static Color hilight(Color c)
	{
		if(c == null) return null;

		double lightness = lightness(c);

		if (lightness >= ((double) 9 / 10))		return(darken(c, (double) 1 / 10));
		else if (lightness <= ((double) 1/5))	return(lighten(c, (double) 6 / 10));
		else  return(lighten(c, (double) 6 / 10));
	}

	/**
	 * Used to calculate a shadow color from a given color.
	 * @param c The color to use in the calculation  
	 * @return the newly calculated shadow color.
	 */
	public static Color shadow(Color c)
	{
		if(c == null) return null;

		double lightness = lightness(c);

		if (lightness >= ((double) 9 / 10))			return(darken(c, (double) 1 / 4));
		else if (lightness <= ((double) 1 / 5))		return(lighten(c, (double) 1 / 5));
		else	return(darken(c, (double) 1 / 4));
	}
	//{{DECLARE_CONTROLS
	//}}
}
