/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.Color;
import java.awt.Panel;
import java.net.URL;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Graphics;
import java.awt.Dimension;
import common.*;

/**
 * ImagePanel displays an image within the panel.
 * The image is specified with a URL.
 */
public class ImagePanel extends java.awt.Panel
{
    /**
     * tiled in this panel.
     */
    public static final int IMAGE_TILED = 0;
    /**
     * centered in this panel.
     */
    public static final int IMAGE_CENTERED = 1;
    /**
     * scaled to fit this panel.
     */
    public static final int IMAGE_SCALED_TO_FIT = 2;
    public static final int IMAGE_MAPPED = 2;    
    /**
     * drawn from the upper left corner of the panel.
     */
    public static final int IMAGE_NORMAL = 3;

    /**
     * Constructs a default ImagePanel. By default the image will be tiled.
     */
     
    public static final int BORDER_NONE = 0; 
    public static final int BORDER_LINE = 1;      
    public int border;

	// Private members

	transient private Image image;
	private URL imageURL;
	private int imageStyle;
     
   
    public int stringX = 0;
    public int stringY = 0;
    public String iString = null;
     
	public ImagePanel()
	{
	    super.setLayout(null);
		image = null;
		border = 0;
		imageStyle = IMAGE_CENTERED;
	}

	public ImagePanel(int b, int s)
	{
	    super.setLayout(null);
		image = null;
		imageStyle = s;
		border = b;
	}

    /**
     * Sets the new panel image style.
     * @param newStyle the new panel image style, one of
     * IMAGE_TILED, IMAGE_CENTERED, or IMAGE_SCALED_TO_FIT
     */
	public void setStyle(int newStyle) { imageStyle = newStyle;	}


    public void setBorder(int b) { border = b; }

	/**
	 * Paints this component using the given graphics context.
     * The graphics context clipping region
     * is set to the bounding rectangle of this component and its [0,0]
     * coordinate is this component's top-left corner.
     *
     * @param g the graphics context used for painting
	 */
	public void update(Graphics g)  { paint(g); }
	 
	public void paint(Graphics g)
	{
		Dimension dim = getSize();
		int panelW = dim.width;
		int panelH = dim.height;

        g.clearRect(0, 0, panelW, panelH);
		
		if (image != null)
		{
			int imageWidth = image.getWidth(this);
			int imageHeight = image.getHeight(this);
			int b;

			switch(imageStyle)
			{
				case IMAGE_TILED:
				{
					//Calculate number of images that should be drawn horizontally
					int numHImages = panelW / imageWidth;
					//Don't forget remainders
					if (panelW % imageWidth != 0) numHImages++;
					//Calculate number of images that should be drawn vertically
					int numVImages = panelH / imageHeight;
					//Don't forget remainders
					if (panelH % imageHeight != 0) 	numVImages++;

					int h;
					int v = 0;
					for (int vCount = 0;vCount < numVImages;vCount++)
					{
						h = 0;
						for (int hCount = 0;hCount < numHImages;hCount++)
						{
							g.drawImage(image, h, v, imageWidth, imageHeight, this);
							h += imageWidth;	//Increment to next column
						}

						v += imageHeight; 	//Increment to next row
					}

					break;
				}

				default:
				case IMAGE_CENTERED:
				{
					int x = (panelW - imageWidth) / 2;
					int y = (panelH - imageHeight) / 2;
					b = 0;
					if(border != BORDER_NONE)
					{
					 g.setColor(Color.black);
					 g.drawRect(x, y, imageWidth - 1, imageHeight - 1);
					 b = 1;
					} 
					
					g.drawImage(image, x + b, y + b, imageWidth - b * 2, imageHeight - b * 2, this);
					break;
				}

				case IMAGE_SCALED_TO_FIT:
				{
					b = 0;
					if(border != BORDER_NONE)
					{
					 g.setColor(Color.black);
					 g.drawRect(0, 0, panelW - 1, panelH - 1);
					 b = 1;
					} 

					g.drawImage(image, b, b, panelW - b * 2, panelH - b * 2, this);
					break;
				}

				case IMAGE_NORMAL:
					g.drawImage(image, 0, 0, this);
					break;
			}//switch
		}
		
 	  if(iString != null)
		{
		 g.drawString(iString, stringX, stringY);   
		}    
		
	  super.paint(g);
	}

    /**
     * Sets the URL of the image to display in this panel.
     * @param url the URL of the image to display
     */
	public void setImage(URL url)
	{
		if (!imageURL.equals(url))
		{
			imageURL = url;
			if (imageURL != null)
			{
				image = getToolkit().getImage(imageURL);
				if (image != null)
				{
					MediaTracker mt = new MediaTracker(this);
					try
					{
						mt.addImage(image, 0);
						mt.waitForAll();
					}
					catch (InterruptedException ie)	{}
				}
			}
			else
			    if(image != null)
			    {
			        image.flush();
			        image = null;
			    }
	        repaint();
		}
	}



	public void setImage(Image img)
	{
       if(img != null)
		{
		  image = img;  
		  MediaTracker mt = new MediaTracker(this);
		  try	{
			 mt.addImage(image, 0);
			 mt.waitForAll();
		  } catch (InterruptedException ie)	{ }
		}
       repaint();
	}


	public void setImage(String name)
	{
		if (name != null) setImage(getToolkit().getImage(name));
		else if(image != null)
   	 {
		   image.flush();
		   image = null;
		   repaint();
		 }
        
	}

    /**
     * Returns the URL of the image being displayed in this panel.
     * @see #setImageURL
     */
	public URL getImageURL()
	{
		return imageURL;
	}

	// Methods

    /**
     * Returns the image being displayed in this panel.
     */
	public Image getImage()
	{
		return image;
	}

    
    /** Dfinit une chane a afficher et sa position */
    
    public void setString(String str, int x, int y)
    {
     stringX = x;
     stringY = y;
     iString = str;
        
    }    

}

