/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;


/**
   A spinner is a component
   with two small direction buttons that lets the user scroll
   a list of predetermined values to select one,
   or possibly enter a new legal value.
 */
public class TSpinner extends Panel implements ActionListener
{
    /** The minimum possible value.  */
    protected int	min;
    /** The maximum possible value. */
    protected int	max;
    /** The current selection value. min <= current <= max. */
    protected int	current;
    /** The text of the current selection. */
    protected String	text;
    /** The text field for this spinner  */
    protected TextField	textField;
    /** Amount the current value chages when scrolling up or down. */
    protected int increment;
    /** The action listener to keep track of listeners for our action event. */
    protected ActionListener actionListener = null;

    protected SpinButtonPanel buttonPanel;
    /** The number of pixels between the text area and the spinner buttons  */
    protected int gap = 2;

    boolean			editable;
    boolean			wrappable;


    /* DGS previous value */
    int previous = 0; 
    /**
     * Construct an empty Spinner. It has the default orientation, an increment
     * of 1, no text field, and minimum, maximum, and current values of 0.
     */
    public TSpinner()
    {
	    super.setLayout(null);
	    
		buttonPanel = new SpinButtonPanel();   
		add(buttonPanel);

        textField = new TextField("0", 1);
        textField.setBackground(Color.white);
        add(textField);
		
        increment = 1;
        min = 0;
        max = 0;
        current = 0; // change textField
        setEditable(true);
        setWrappable(false);
        
        previous = 0;
        
		buttonPanel.addActionListener(this);
		textField.addActionListener(this);
        
    }

    public TSpinner(int current, int min, int max)
    {
      this();  
      setMin(min);
      setMax(max);
      setCurrent(current);
      previous = current;
    }

    
   public void setBounds(int x, int y, int w, int h)
   {
     super.setBounds(x, y, w, h);
     
   	 buttonPanel.setBounds(w - 20 , 0, 20, 20);
     textField.setBounds(0,0, w - 20, 20);
   }   
    
    /**
     * Conditionally enables editing of the Spinner's TextField.
     */
    public void setEditable(boolean f)
    {
      editable = f;
      textField.setEditable(f);
    }

    /**
     * Returns whether the Spinner's TextField is editable.
     */
    public boolean isEditable()  {  return editable; }

    /**
     * Conditionally sets whether the current value wraps around from maximum
     * to minimum and from minimum to maximum.
     * @param f if true the value can wrap
     */
    public void setWrappable(boolean f)
    {
      wrappable = f;
      updateButtonStatus();
    }

    /**
     * Gets whether the current value can wrap around from maximum
     * to minimum and from minimum to maximum.
     */
    public boolean isWrappable() { return wrappable; }

    /**
     * Sets the minimum value the spinner may have.
     */
    public void setMin(int i)
    {
       min = i;
       setCurrent(current);
       updateButtonStatus();
	}

    /**
     * Gets the current minimum value the spinner may have.
     */
    public int getMin() { return min;  }

    /**
     * Sets the maximum value the spinner may have.
     */
    public void setMax(int i)
    {
        max = i;
        setCurrent(current);
    	updateButtonStatus();
    }

    /**
     * Gets the current maximum value the spinner may have.
     */
    public int getMax() { return max;  }

    /**
     * Sets the value of the spinner.
     */
    public void setCurrent(int i)
    {
      current = i;
      if(current < min) current = min;
      if(current > max) current = max;
      setText(current);
      updateButtonStatus();
    }

    /** Gets the current value of the spinner */
    public int getCurrent() { return current;  }


    /**
     * Sets whether the spinner buttons will continually post notify events
     * while pressed.
     */
    public void setNotifyWhilePressed(boolean f)
    {
        buttonPanel.setNotifyWhilePressed(f);
    }

    /**
     * Gets the current notifyWhilePressed status.
     */
    public boolean isNotifyWhilePressed()
    {
        return buttonPanel.isNotifyWhilePressed();
    }


    /**
     * Sets the notification event delay of the spinner buttons in milliseconds.
     */
    public void setDelay(int d)
    {
        buttonPanel.setDelay(d);
    }

    /**
     * Returns the current delay between notification events of the spinner
     * buttons in milliseconds.
     */
    public int getDelay() { return buttonPanel.getDelay(); }

	/**
	 * Since layout managers CANNOT BE USED with this container the standard
	 * setLayout has been OVERRIDDEN for this container and does nothing.
	 */
    public void setLayout(LayoutManager lm)  {  }



	/**
	 * Returns the recommended dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the recommended size of this component.
	 */
    public Dimension getPreferredSize()
    {
	 	//!!! LAB !!!	When TextField returns correct prefered size information and the x,y positioning
	 	//				is fixed, remove Mac specific code.

    	int width = 50;
    	int height = 20;

	    Dimension t = textField.getPreferredSize();
	    Dimension b = buttonPanel.getPreferredSize();
	    int macTextAdjustment = 0;
	    int macPosAdjustment = 0;
/*
		    if(OS.isMacintosh())
		    {
		    	macPosAdjustment = 3;
		    	macTextAdjustment = 2;
		    }
*/
	    width	= b.width + t.width + gap + macPosAdjustment;
	    height	= t.height + macTextAdjustment;

		return new Dimension(width, height);
    }

    /**
	 * Returns the minimum dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the minimum size of this component.
     * It simply returns the results of a call to preferedSize().
	 */
    public Dimension getMinimumSize()
    {
        return getPreferredSize();
    }


  	/**
     * Adds the specified action listener to receive action events
     * from this button.
     */
	public void addActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.add(actionListener, l);
	}

    /**
     * Removes the specified action listener so it no longer receives
     * action events from this component.
     */
	public void removeActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, l);
	}



    /**
     * Is the given value valid for the Current property .
     */
    protected boolean isValidCurrent(int i)
    {
        return !(i > max || i < min);
    }

    /**
     * Is the given value valid for the Max property .
     */
    protected boolean isValidMax(int i) { return (i >= min);}

    /**
     * Is the given value valid for the Min property .
     */
    protected boolean isValidMin(int i) { return (i <= max); }

	/**
	* Fire an action event to the listeners
	*/
	protected void sourceActionEvent( String s)
	{
		if (actionListener != null)
			actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, s));
	}

    /**
     * Increments the spinner's value and handles wrapping as needed.
     */
    protected void scrollUp()
    {
       int i = current + increment;
   	   
   	   if(i > max)
   	   {
        if (wrappable) i = min; 
            else	   i = max; 
       }
	  
	  setCurrent(i);
    }

    /**
     * Decrements the spinner's value and handles wrapping as needed.
     */
    protected void scrollDown()
    {
      int i = current - increment;        
      if(i < min)
      {
        if (wrappable) i = max; else i = min;
      }
    		
   	  setCurrent(i);
    }

    /**
       DGS
       Change affichage, remplace updateText
    */   
    public void setText(int val)
    {
//   System.out.println("val " + val);   
     textField.setText(Integer.toString(val));   
    }    

    /**
     * Returns the text that is in the entry TextField.
     */
	public String getText() { return textField.getText(); }


    /**
       DGS
       Test if change
    */
    

    public boolean isModified()
    {
     boolean test = previous != current;   
     previous = current;
     return test;
    }    


	/**
	 * Handles enabling or disabling the spinner buttons as needed.
	 */
	protected void updateButtonStatus()
	{
	 if(buttonPanel != null)
      {
	   if(wrappable)
	    {
		 buttonPanel.setUpButtonEnabled(true);
		 buttonPanel.setDownButtonEnabled(true);
		}
	    else
		{
		if(current == max && current == min)
		{
	     buttonPanel.setUpButtonEnabled(false);
	     buttonPanel.setDownButtonEnabled(false);
		}
	    else if(current == max)
	    {
	     buttonPanel.setUpButtonEnabled(false);
	     buttonPanel.setDownButtonEnabled(true);
	    }
	    else if(current == min)
	    {
	     buttonPanel.setUpButtonEnabled(true);
	     buttonPanel.setDownButtonEnabled(false);
	    }
	    else
	    {
		 buttonPanel.setUpButtonEnabled(true);
		 buttonPanel.setDownButtonEnabled(true);
		}
	   }
	  }
	}




	  public void actionPerformed(ActionEvent e)
	  {
		String actionCommand = e.getActionCommand();

		if(actionCommand.equals("Increment"))
		{
		  scrollUp();
		  sourceActionEvent("ScrollUp");
		}
		else if(actionCommand.equals("Decrement"))
		{
		  scrollDown();
		  sourceActionEvent("ScrollDown");
		}
		else	
			
 		//if(actionCommand.equals("Enter")) // valeur entr
 		// if (((TextField) e.getSource()) == textField)  // pas propre
   		{
   		  setCurrent(new Integer(textField.getText()).intValue());
          sourceActionEvent("Enter");
   		  return;
   		}
			
	  }

}


