/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.util.*;
import common.*;

public class VectorInt
{
    /**
     * The array buffer into which the components of the VectorInt are
     * stored. The capacity of the VectorInt is the length of this array buffer.
     */

    public int elementData[];

    /** The number of valid components in the VectorInt  */
    public int elementCount;

    /**
     * The amount by which the capacity of the VectorInt is automatically
     * incremented when its size becomes greater than its capacity. If
     * the capacity is <code>0</code>, the capacity of the VectorInt is
     * doubled each time it needs to grow.
     */
    protected int capacityIncrement;


    /**
     * Constructs an empty VectorInt with the specified initial capacity and
     * capacity increment.
     */

    public VectorInt(int initialCapacity, int capacityIncrement)
    {
	 super();
	 this.elementData = new int[initialCapacity];
	 this.capacityIncrement = capacityIncrement;
    		//{{INIT_CONTROLS
		//}}
}

    /**
     * Constructs an empty VectorInt with the specified initial capacity.
     */

    public VectorInt(int initialCapacity) { this(initialCapacity, 1); }

    /** Constructs an empty VectorInt   */

    public VectorInt() { this(16); }

    /**
     * Copies the components of this VectorInt into the specified array.
     * The array must be big enough to hold all the objects in this  VectorInt.
     */

    public final synchronized void copyInto(int anArray[])
    {
	 int i = elementCount;

	 while (i-- > 0) {  anArray[i] = elementData[i]; }
    }

    /**
     * Trims the capacity of this VectorInt to be the VectorInt's current
     * size. An application can use this operation to minimize the
     * storage of a VectorInt.
     */

    public final synchronized void trimToSize()
    {
	 int oldCapacity = elementData.length;
	 if (elementCount < oldCapacity)
	  {
	    int oldData[] = elementData;
	    elementData = new int[elementCount];
	    copy(oldData, elementData, elementCount);
	  }
    }

    /**
     * Increases the capacity of this VectorInt, if necessary, to ensure
     * that it can hold at least the number of components specified by
     * the minimum capacity argument.
     */

    public final synchronized void ensureCapacity(int minCapacity)
    {
	 int oldCapacity = elementData.length;
	 if (minCapacity > oldCapacity)
	  {
	    int oldData[] = elementData;
	    int newCapacity = (capacityIncrement > 0) ?
		(oldCapacity + capacityIncrement) : (oldCapacity * 2);

        if (newCapacity < minCapacity) { newCapacity = minCapacity; }

	    elementData = new int[newCapacity];
        copy(oldData, elementData, elementCount);	    
	  }
    }

    /**
     * Sets the size of this VectorInt. If the new size is greater than the
     * current size, new <code>null</code> items are added to the end of
     * the VectorInt. If the new size is less than the current size, all
     * components at index <code>newSize</code> and greater are discarded.
     */

    public final synchronized void setSize(int newSize)
    {
	 if (newSize > elementCount) { ensureCapacity(newSize); }
	 else
	  {
	    for (int i = newSize ; i < elementCount ; i++)
		  elementData[i] = 0;
	  }
	 elementCount = newSize;
    }

    /** Returns the current capacity of this VectorInt */

    public final int capacity() { return elementData.length; }

    /** Returns the number of components in this VectorInt */

    public final int size() { return elementCount; }

    /**
     * Tests if this VectorInt has no components.
     */

    public final boolean isEmpty() { return elementCount == 0; }


   /** Tests if the specified object is a component in this VectorInt  */

    public final boolean contains(int elem) { return indexOf(elem, 0) >= 0;}

    /**
     * Searches for the first occurence of the given argument, testing
     * for equality using the <code>equals</code> method.
     */

    public final int indexOf(int elem) { return indexOf(elem, 0); }

    /**
     * Searches for the first occurence of the given argument, beginning
     * the search at <code>index</code>, and testing for equality using
     * the <code>equals</code> method.
     */

    public final synchronized int indexOf(int elem, int index)
    {

	 for (int i = index ; i < elementCount ; i++)
	   if (elem == elementData[i]) return i;

     return -1;
    }

    /**
     * Returns the index of the last occurrence of the specified object in
     * this VectorInt.
     */

    public final int lastIndexOf(int elem)
    {
	   return lastIndexOf(elem, elementCount-1);
    }

    /**
     * Searches backwards for the specified object, starting from the
     * specified index, and returns an index to it.
     */

    public final synchronized int lastIndexOf(int elem, int index)
    {
      for (int i = index ; i >= 0 ; i--)
	    if (elem == elementData[i]) return i;

	  return -1;
    }

    /** Returns the component at the specified index  */

    public final synchronized int elementAt(int index)
    {
	  if (index >= elementCount)
	    throw new ArrayIndexOutOfBoundsException(index + " >= " + elementCount);

      return elementData[index];
    }

    /** Returns the first component of this VectorInt  */

    public final synchronized int firstElement()
    {
	 if(elementCount == 0) throw new NoSuchElementException();
	 return elementData[0];
    }

    /** Returns the last component of the VectorInt  */

    public final synchronized int lastElement()
    {
	 if (elementCount == 0) throw new NoSuchElementException();
	 return elementData[elementCount - 1];
    }

    /**
     * Sets the component at the specified <code>index</code> of this
     * VectorInt to be the specified object. The previous component at that
     * position is discarded.
     */
    public final synchronized void setElementAt(int val, int index)
    {
 	  if (index >= elementCount)
	    throw new ArrayIndexOutOfBoundsException(index + " >= " + elementCount);

	  elementData[index] = val;
    }

    /**
     * Deletes the component at the specified index. Each component in
     * this VectorInt with an index greater or equal to the specified
     * <code>index</code> is shifted downward to have an index one
     * smaller than the value it had previously.
     * <p>
     * The index must be a value greater than or equal to <code>0</code>
     * and less than the current size of the VectorInt.
     */

    public final synchronized void removeElementAt(int index)
    {
	 if (index >= elementCount)
	  throw new ArrayIndexOutOfBoundsException(index + " >= " + elementCount);
	 else
	 if (index < 0)
      throw new ArrayIndexOutOfBoundsException(index);

	 int j = elementCount - index - 1;
	 if (j > 0) copy(elementData, index + 1, elementData, index, j);
	 elementCount--;
	 elementData[elementCount] = 0;
    }

    /**
     * Inserts the specified object as a component in this VectorInt at the
     * specified <code>index</code>. Each component in this VectorInt with
     * an index greater or equal to the specified <code>index</code> is
     * shifted upward to have an index one greater than the value it had
     * previously.
     * <p>
     * The index must be a value greater than or equal to <code>0</code>
     * and less than or equal to the current size of the VectorInt.
     */

    public final synchronized void insertElementAt(int obj, int index) 
    {
	if (index >= elementCount + 1) 
     throw new ArrayIndexOutOfBoundsException(index + " > " + elementCount);
	
	ensureCapacity(elementCount + 1);
	copy(elementData, index, elementData, index + 1, elementCount - index);
	elementData[index] = obj;
	elementCount++;
    }

    /**
     * Adds the specified component to the end of this VectorInt,
     * increasing its size by one. The capacity of this VectorInt is
     * increased if its size becomes greater than its capacity.
     */

     public final synchronized void addElement(int val)
     {
	  ensureCapacity(elementCount + 1);
	  elementData[elementCount++] = val;
     }

    /**
     * Removes the first occurrence of the argument from this VectorInt. If
     * the object is found in this VectorInt, each component in the VectorInt
     * with an index greater or equal to the object's index is shifted
     * downward to have an index one smaller than the value it had previously.
     */

    public final synchronized boolean removeElement(int obj)
    {
  	 int i = indexOf(obj);
	 
	 if (i >= 0) { removeElementAt(i); return true;}
	 return false;
    }

    /**
     * Removes all components from this VectorInt and sets its size to zero.
     */

    public final synchronized void removeAllElements()
    {
	 for (int i = 0; i < elementCount; i++)  elementData[i] = 0;
	 elementCount = 0;
    }


    /**
     * Returns a string representation of this VectorInt.
     */

    public final synchronized String toString()
    {
	 int max = size() - 1;
	 StringBuffer buf = new StringBuffer();

	 buf.append("[");

	 for (int i = 0 ; i <= max ; i++)
	 {
	   buf.append((new Integer(elementData[i])).toString());
	   if (i < max) { buf.append(", ");	 }
	}
	 buf.append("]");
	 
	 return buf.toString();
    }
    
 
    
    static public void copy(int [] src, int [] dst, int len)
    {
     for (int i = 0; i < len; i++)  dst[i] = src[i];
     //System.arraycopy(src, 0, dst, 0, len);     
    }

    static public void copy(int [] src, int sdeb, int [] dst, int ddeb, int len)
    {
     for (int i = 0; i < len; i++)  dst[i + ddeb] = src[i + sdeb];
     //System.arraycopy(src, sdeb, dst, ddeb, len);          
    }
    
    
    // Enumerateur

    public int index = 0;

    public void restart() { index = 0; }
    
    public boolean hasMoreElements() { return index < elementCount; }    

    public int nextElement()
    {
     if (index < elementCount) return elementData[index++];
     else throw new NoSuchElementException("IntEnumerator");
    }
    
	//{{DECLARE_CONTROLS
	//}}
}
