/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.util.*;
import common.Sort;


public class VectorString
{
   /**
    * Buffer into which the strings are stored.
    * The capacity of the VectorString is the length of this array buffer.
    */

   public String elementData[];

   public int elementCount;    /** The number of valid components */

    /**
     * The amount by which the capacity of the VectorString is automatically
     * incremented when its size becomes greater than its capacity.
     * If the capacity is <code>0</code>, the capacity of the VectorString is
     * doubled each time it needs to grow.
     */
    public int capacityIncrement;
    public int initialCapacity;

    /**
     * Constructs an empty VectorString with the specified capacity/increment.
     */

    public VectorString(int initialCapacity, int cI)
    {
	 //super();
	 this.elementData = new String[initialCapacity];
	 this.capacityIncrement = cI;
     this.initialCapacity = initialCapacity;
     //{{INIT_CONTROLS
		//}}
}

    /** Constructs with the specified initial capacity */

    public VectorString(int initialCapacity) { this(initialCapacity, 1); }

    /** Constructs an empty VectorString, initial 16  */

    public VectorString() { this(16); }

    /** DGS remet  zro */
    public void clear()
    {
	 this.elementData = new String[initialCapacity];
     elementCount = 0;
    }

    /**
     * Copies the components of this VectorString into the specified array.
     * The array must be big enough to hold all the objects in this  VectorString.
     */

    public final synchronized void copyInto(String anArray[])
    {
	 int i = elementCount;

	 while (i-- > 0) {  anArray[i] = elementData[i]; }
    }


    public final synchronized void copyInto(VectorString vs)
    {
	 int i = elementCount;
	 //int ci = vs.capacityIncrement;

	 vs.setSize(i);  // On ne peut reconstruire, ce serait local
	 for(int j = 0; j < i; j++) {  vs.addElement(elementData[j]); }
    }


    /**
        Retourne le tableau de chanes
        Si on retournait directement elementData
        le tableau rcepteur serait dimensionn  initialCapacity
      */
    public final synchronized String[] toArray()
    {
      String dst[] = new String[elementCount];
      System.arraycopy(elementData, 0, dst, 0, elementCount);
      return dst;
    }

    /**
       Retourne directement elementData
       Il faudra utiliser elementCount du VectorString
       et non array.length
      */
    public final synchronized String[] getArray() { return elementData;}

    /**
     * Trims the capacity of this VectorString to be the
     * VectorString's current size.
     * An application can use this operation to minimize the
     * storage of a VectorString.
     */
    // Supprime les emplacements inutiliss

    public final synchronized void trimToSize()
    {
	 int oldCapacity = elementData.length;
	 if (elementCount < oldCapacity)
	  {
	    String oldData[] = elementData;
	    elementData = new String[elementCount];
	    copy(oldData, elementData, elementCount);
	  }
    }

    /**
     * Increases the capacity of this VectorString, if necessary, to ensure
     * that it can hold at least the number of components specified by
     * the minimum capacity argument.
     */

    public final synchronized void ensureCapacity(int minCapacity)
    {
	 int oldCapacity = elementData.length;

	 if (minCapacity > oldCapacity)
	  {
	    String oldData[] = elementData;
	    int newCapacity = (capacityIncrement > 0) ?
		(oldCapacity + capacityIncrement) : (oldCapacity * 2);

        if (newCapacity < minCapacity) { newCapacity = minCapacity; }

	    elementData = new String[newCapacity];
        copy(oldData, elementData, elementCount);
	 }
    }

    /**
     * Sets the size of this VectorString. If the new size is greater than the
     * current size, new <code>null</code> items are added to the end of
     * the VectorString. If the new size is less than the current size, all
     * components at index <code>newSize</code> and greater are discarded.
     */

    public final synchronized void setSize(int newSize)
    {
	 if (newSize > elementCount) { ensureCapacity(newSize); }
	 else
	  {
	    for (int i = newSize ; i < elementCount ; i++)
	    {
		  elementData[i] = null;
	    }
	  }
	 elementCount = newSize;
    }

    /** Returns the current capacity of this VectorString */

    public final int capacity() { return elementData.length; }

    /** Returns the number of components in this VectorString */

    public final int size() { return elementCount; }

    /** Tests if this VectorString has no components  */

    public final boolean isEmpty() { return elementCount == 0; }


   /** Tests if the specified object is a component in this VectorString  */

    public final boolean contains(String elem)
    {
 	   return indexOf(elem, 0) >= 0;
    }

    /**
     * Searches for the first occurence of the given argument, testing
     * for equality using the <code>equals</code> method.
     */

    public final int indexOf(String elem) { return indexOf(elem, 0); }

    /**
     * Searches for the first occurence of the given argument, beginning
     * the search at <code>index</code>, and testing for equality using
     * the <code>equals</code> method.
     */

    public final synchronized int indexOf(String elem, int index)
    {

	 for (int i = index ; i < elementCount ; i++)
	   if (elem.equals(elementData[i])) return i;

     return -1;
    }

    /**
     * Returns the index of the last occurrence of the specified object in
     * this VectorString.
     */

    public final int lastIndexOf(String elem)
    {
	   return lastIndexOf(elem, elementCount-1);
    }

    /**
     * Searches backwards for the specified object, starting from the
     * specified index, and returns an index to it.
     */

    public final synchronized int lastIndexOf(String elem, int index)
    {
      for (int i = index ; i >= 0 ; i--)
      {
	    if (elem.equals(elementData[i])) return i;
      }
	  return -1;
    }

    /** Returns the component at the specified index  */

    public final synchronized String elementAt(int index)
    {
	  if (index >= elementCount)
	    throw new ArrayIndexOutOfBoundsException(index + " >= " + elementCount);

      return elementData[index];
    }

    /** Returns the first component of this VectorString  */

    public final synchronized String firstElement()
    {
 	  if (elementCount == 0) throw new NoSuchElementException();
	  return elementData[0];
    }

    /** Returns the last component of the VectorString  */

    public final synchronized String lastElement()
    {
 	  if (elementCount == 0) throw new NoSuchElementException();
	  return elementData[elementCount - 1];
    }

    /**
     * Sets the component at the specified <code>index</code> of this
     * VectorString to be the specified object. The previous component at that
     * position is discarded.
     */
    public final synchronized void setElementAt(String s, int index)
    {
 	  if (index >= elementCount)
	   throw new ArrayIndexOutOfBoundsException(index + " >= " +  elementCount);

	  elementData[index] = s;
    }

    /**
     * Deletes the component at the specified index. Each component in
     * this VectorString with an index greater or equal to the specified
     * <code>index</code> is shifted downward to have an index one
     * smaller than the value it had previously.
     * <p>
     * The index must be a value greater than or equal to <code>0</code>
     * and less than the current size of the VectorString.
     */

    public final synchronized void removeElementAt(int index)
    {
	 if (index >= elementCount)
	 {
      throw new ArrayIndexOutOfBoundsException(index + " >= " + elementCount);
	 }
	 else
	 if (index < 0)
	    throw new ArrayIndexOutOfBoundsException(index);

	 int j = elementCount - index - 1;
	 if (j > 0) copy(elementData, index + 1, elementData, index, j);

	 elementCount--;
	 elementData[elementCount] = null; /* to let gc do its work */
    }

    /**
     * Inserts the specified object as a component in this VectorString at the
     * specified <code>index</code>. Each component in this VectorString with
     * an index greater or equal to the specified <code>index</code> is
     * shifted upward to have an index one greater than the value it had
     * previously.
     * <p>
     * The index must be a value greater than or equal to <code>0</code>
     * and less than or equal to the current size of the VectorString.
     */

    public final synchronized void insertElementAt(String obj, int index) {
	if (index >= elementCount + 1)
	{
	 throw new ArrayIndexOutOfBoundsException(index + " > " + elementCount);
	}
	ensureCapacity(elementCount + 1);
	copy(elementData, index, elementData, index + 1, elementCount - index);
	elementData[index] = obj;
	elementCount++;
    }

    /**
     * Adds the specified component to the end of this VectorString,
     * increasing its size by one. The capacity of this VectorString is
     * increased if its size becomes greater than its capacity.
     */

     public final synchronized void addElement(String s)
     {
	  ensureCapacity(elementCount + 1);
	  elementData[elementCount++] = s;
     }

     public final synchronized void addElement(StringBuffer sb)
     {
	  ensureCapacity(elementCount + 1);
	  elementData[elementCount++] = new String(sb);
     }

     public final synchronized boolean addUnicElement(String s)
     {
       int i = 0;
       while(i < elementCount)
        if(s.equalsIgnoreCase(elementData[i++])) return false;

       addElement(s);
       return true;
     }

    /**
     * Removes the first occurrence of the argument from this VectorString. If
     * the object is found in this VectorString, each component in the VectorString
     * with an index greater or equal to the object's index is shifted
     * downward to have an index one smaller than the value it had previously.
     */

    public final synchronized boolean removeElement(String obj)
    {
 	 int i = indexOf(obj);

	 if(i >= 0) { removeElementAt(i); return true; }
	 return false;
    }

    /** Removes all components from this VectorString
     *  and sets its size to zero.
     */

    public final synchronized void removeAllElements()
    {
	 for (int i = 0; i < elementCount; i++)  elementData[i] = null;
	 elementCount = 0;
    }



    /** Returns a string representation of this VectorString  */

    public final synchronized String toString()
    {
	 int max = size() - 1;
	 StringBuffer buf = new StringBuffer();

	 buf.append("[");
	 for (int i = 0 ; i <= max ; i++)
	 {
	   buf.append(elementAt(i));
	   if (i < max) { buf.append(", "); }
	 }
	 buf.append("]");

	 return buf.toString();
    }


    /**
      Compare deux entres en diffrenciant les majuscules
      */

    public int compare(int ia, int ib)
    {
      return elementData[ia].compareTo(elementData[ib]);
    }


 /**
     Tri et Gestion de liste de rpertoire
    */

  boolean separeDrives = false;

  void swap(int a, int b)
  {
    String s = elementData[a];
    elementData[a] = elementData[b];
    elementData[b] = s;
  }

  int sortCompare(int a, int b)
  {
    String sa = elementData[a];
    String sb = elementData[b];

    if(separeDrives)
    {
    boolean aunit = (sa.charAt(1) == ':') && (sa.length() == 2);
    boolean bunit = (sb.charAt(1) == ':') && (sb.length() == 2);
    if(aunit != bunit)  // si tous deux ou aucun, on continue
     {
      if(aunit) return -1; // si seul a unit, infrieur
      return 1;            // si seul b unite, suprieur
     }
    }
    return Sort.superCompare(sa, sb);
  }


  public int separeDrives()
  {
   return SortDir.separeDrives(elementData, 0, elementCount);
  }

  /**
    Sort insertion
   */

  public void sort(boolean separe)  { sort(0, size(), separe); }
  public void sort() { sort(0, size()); }
  public void sort(int s, int l, boolean separe)
  {
   separeDrives = true;
   sort(s, l);
   separeDrives = false;
  }

  public void sort(int start, int limit)
  {
   int i, j;

   if(limit <= start) return;

   j = start;
   i = j + 1;
   while(i < limit)
   {
     while((j >= start) && sortCompare(j, j + 1) > 0)
      {
       swap(j, j + 1);
       j--;
      }

     j = i;
     i++;
    } // jusqu' lim

	return;
  }



    /**
      Copy static
      */


    static public void copy(String [] src, String [] dst, int len)
    {
     for (int i = 0; i < len; i++)  dst[i] = src[i];
     //System.arraycopy(src, 0, dst, 0, len);
    }

    static public void copy(String [] src, int sdeb, String [] dst, int ddeb, int len)
    {
     for (int i = 0; i < len; i++)  dst[i + ddeb] = src[i + sdeb];
     //System.arraycopy(src, sdeb, dst, ddeb, len);
    }


    // Enumerateur

    public int index = 0;   // Pour numration

    public void restart() { index = 0; }

    public boolean hasMoreElements() { return index < elementCount; }

    public String nextElement()
    {
     if (index < elementCount) return elementData[index++];
     else throw new NoSuchElementException("StringEnumerator");
    }

	//{{DECLARE_CONTROLS
	//}}
}



